
package Core.CubeConfiguration;

import WorkSpace.CubeConfiguration.MyDefaultMutableTreeNode;
import WorkSpace.CubeConfiguration.MyJDesktopPane;
import java.io.*;
import java.sql.Connection;
import java.sql.ResultSet;
import java.sql.ResultSetMetaData;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.*;
import javax.swing.JTable;
import javax.swing.JTree;
import javax.swing.table.DefaultTableModel;
import javax.swing.tree.DefaultTreeModel;
import nu.xom.*;


public class CubeSchemaWriter
{ 
       String saveLocation,cubeName;
       JTable tbl_measures;
       JTree tre_dimension;
       Vector joins1,joins,snowjoins,levelTables;       
       Connection connection,con;
       MeasureType measures[];
       MyJDesktopPane pnl_contenedor;
       String Dimensiontable="";
       String XMLFileName="";
       String username;
       String url;
       FileStorage f;
       boolean xmlerror=true;

    public CubeSchemaWriter(String saveLocation, String cubeName, JTable tbl_measures, JTree tre_dimension, MyJDesktopPane pnl_contenedor, Connection connection,Connection con,String username,String url) {
        this.saveLocation=saveLocation;
        this.username=username;
        this.cubeName=cubeName;
        this.url=url;        
        this.tbl_measures=tbl_measures;
        this.tre_dimension=tre_dimension;
        this.connection=connection;
        this.con=con;
        this.pnl_contenedor=pnl_contenedor;
        createMeasures();
        getXMLString();
        createCubeSchema();
       }
    
  private File init(String XMLFileName) {
    File xmlfile = new File(System.getProperty("user.dir"));
    String xmlpath = xmlfile.getAbsolutePath() + saveLocation;
    File xmlpathfile = new File(xmlpath);
    File jdbcFile = null;
    if (!(xmlpathfile.exists()))
      try {
        xmlpathfile.mkdirs();
        jdbcFile = new File(xmlpathfile.getAbsolutePath() + "/" + XMLFileName);
        jdbcFile.createNewFile();
      } catch (IOException ex) {
        ex.printStackTrace();
      }

    else
      jdbcFile = new File(xmlpathfile.getAbsolutePath() + "/" + XMLFileName);
    return jdbcFile;
  }
    public void createCubeSchema()
    {
        try {
            OutputStream out;            
            out = null;            
            XMLFileName = (new StringBuilder()).append(cubeName).append(".xml").toString();
            File file =init(XMLFileName);
            out = new FileOutputStream(file);
            Serializer serializer = new Serializer(out, "UTF-8");
            serializer.setIndent(4);
            serializer.setLineSeparator("\r\n");
            serializer.setPreserveBaseURI(false);
            serializer.write(xmlDoc);
            serializer.flush();
            if (out != null) {
                try {
                    out.close();
                } 
                catch (IOException ex) {
                    ex.printStackTrace();
                }
            }

        } catch (Exception ex) {
          
        }
    }

   
 private Element createElement(String elemName)
    {
        return new Element(elemName);
    }
     private Attribute createAttribute(String attribName, String attribVal)
    {
        return new Attribute(attribName, attribVal);
    }
     
   public void getXMLString()
    {
        try {            
            Element root = new Element("Schema");
            root.addAttribute(new Attribute("name", cubeName));
            xmlDoc = new Document(root);
            Element child = new Element("Cube");
            child.addAttribute(new Attribute("name", cubeName));
            root.appendChild(child);
            if (measures.length > 0) {                
                factTableName = measures[0].getTableName();
                Element tblElem = createElement("Table");
                tblElem.addAttribute(createAttribute("name", factTableName));
                child.appendChild(tblElem);
            }            
            createDimensionElements(child);
            for (int j = 0; j < measures.length; j++) {
                MeasureType measureObj = measures[j];
                Element measureChild = new Element("Measure");
                measureChild.addAttribute(createAttribute("name", measureObj.getDataFieldName()));
                String measureAttribColumn = measureObj.getSourceField();
                measureAttribColumn = measureAttribColumn.substring(measureAttribColumn.indexOf(".") + 1);
                measureChild.addAttribute(createAttribute("column", measureAttribColumn));
                measureChild.addAttribute(createAttribute("aggregator", measureObj.getSummarizeBy().toLowerCase()));
                measureChild.addAttribute(createAttribute("datatype", measureObj.getTypeName()));
                measureChild.addAttribute(createAttribute("formatString", measureObj.getFormat()));
                child.appendChild(measureChild);
            }
        } catch (Exception ex) {
            
        
        }

    }
  
   
       private Element createDimensionElement( Element parent,String elemName)
    {
        Element childElem = createElement(elemName);
        parent.appendChild(childElem);
        return childElem;
    }
   
    private void createDimensionElements(Element child)
    {   DefaultTreeModel modelo =(DefaultTreeModel)this.tre_dimension.getModel();
        MyDefaultMutableTreeNode query1=(MyDefaultMutableTreeNode)modelo.getRoot();
         int k=query1.getChildCount();
        for(int i = 0; i < k; i++)
        {   Element dimElem = createDimensionElement(child,"Dimension");
            MyDefaultMutableTreeNode nodo=(MyDefaultMutableTreeNode)query1.getChildAt(i);
            dimElem.addAttribute(createAttribute("name",nodo.getName()));           
            Dimensiontable=nodo.getNameColumn().substring(0,nodo.getNameColumn().indexOf("."));            
            if(!Dimensiontable.equalsIgnoreCase(factTableName))
            {
                String keyColName = getDimensionForeignKey(factTableName,Dimensiontable);
                if(keyColName != null && keyColName.length() > 0)
                    dimElem.addAttribute(createAttribute("foreignKey", keyColName));
            }
            else{xmlerror=false;}            
            createHierarchyElement(nodo, dimElem);
            
        }
    }
    private void createHierarchyElement(MyDefaultMutableTreeNode hier, Element dimensionElement)                {        
            snowjoins=new Vector();
            MyDefaultMutableTreeNode hieraux;            
            for(int d=0;d<hier.getChildCount();d++)
            {   
                hieraux=(MyDefaultMutableTreeNode) hier.getChildAt(d);
                Element hierarchyElem = new Element("Hierarchy");
                hierarchyElem.addAttribute(createAttribute("name", hieraux.getName()));
                hierarchyElem.addAttribute(createAttribute("hasAll", "true"));
                String allMemberName = (new StringBuilder()).append("All ").append(hieraux.getName().toLowerCase()).toString();
                hierarchyElem.addAttribute(createAttribute("allMemberName", allMemberName));
                if(!Dimensiontable.equalsIgnoreCase(factTableName))
                {
                    String keyColName = getHierarchyPrimaryKey(factTableName,hier.getNameColumn().substring(0,hier.getNameColumn().indexOf(".")));
                    if(keyColName != null && keyColName.length() > 0)
                        hierarchyElem.addAttribute(createAttribute("primaryKey", keyColName));
                }
                dimensionElement.appendChild(hierarchyElem);
                if(isSnowFlake(hieraux))
                {   hierarchyElem.addAttribute(createAttribute("primaryKeyTable", hier.getNameColumn().substring(0,hier.getNameColumn().indexOf("."))));
                    snowjoins =getHierarchyTableJoins(hieraux);
                    createJoinElement(snowjoins, 0,0, hierarchyElem);
                } else
                {
                    Element tblElem = createElement("Table");
                    tblElem.addAttribute(createAttribute("name", hier.getNameColumn().substring(0,hier.getNameColumn().indexOf("."))));
                    hierarchyElem.appendChild(tblElem);
                }
                createLevelElements(hieraux,hierarchyElem);
            }
    }

    private void createJoinElement(Vector snowjoins, int index,int index1, Element parent)
    {
               
        String leftColumn = snowjoins.get(index).toString();
        index++;
        String rightColumn = snowjoins.get(index).toString();
        Element joinElem = new Element("Join");
        joinElem.addAttribute(createAttribute("leftKey", rightColumn));
        if(index < snowjoins.size() - 1)
            joinElem.addAttribute(createAttribute("rightAlias", levelTables.get(index1+1).toString()));
        joinElem.addAttribute(createAttribute("rightKey", leftColumn));
        parent.appendChild(joinElem);
        Element tableElem = new Element("Table");
        tableElem.addAttribute(createAttribute("name", levelTables.get(index1).toString()));
        joinElem.appendChild(tableElem);
        if(index < snowjoins.size() - 1)
        {
            createJoinElement(snowjoins, ++index,++index1, joinElem);
        } else
        {
            tableElem = new Element("Table");
            tableElem.addAttribute(createAttribute("name", levelTables.get(index1+1).toString()));
            joinElem.appendChild(tableElem);
        }
    }

    private void createLevelElements(MyDefaultMutableTreeNode hier,Element parent)
    {  MyDefaultMutableTreeNode nodop= new MyDefaultMutableTreeNode();
        nodop=hier;
        for(int i = 0; i < hier.getDepth(); i++)
        {   MyDefaultMutableTreeNode nodoh=(MyDefaultMutableTreeNode)nodop.getChildAt(0);                        
            Element levelElement = createDimensionElement(parent, "Level");
            levelElement.addAttribute(createAttribute("name", nodoh.getName()));
            levelElement.addAttribute(createAttribute("table",  nodoh.getNameColumn().substring(0,nodoh.getNameColumn().indexOf("."))));
            String levelAttribColumn = nodoh.getNameColumn();
            levelAttribColumn = levelAttribColumn.substring(levelAttribColumn.indexOf(".") + 1);
            levelElement.addAttribute(createAttribute("column", levelAttribColumn));
            levelElement.addAttribute(createAttribute("uniqueMembers", nodoh.getUniqueMenbers()));
            compruebaType(nodoh,levelElement);
            nodop=nodoh;
        }

    }
    
   private boolean isSnowFlake(MyDefaultMutableTreeNode hier)
    {
        boolean isSnowFlake = false;
        String hierarchyTableName =hier.getNameColumn().substring(0,hier.getNameColumn().indexOf("."));        
        MyDefaultMutableTreeNode nodop= new MyDefaultMutableTreeNode();
        nodop=hier;
       for(int i = 0; i < hier.getDepth() && !isSnowFlake; i++)
       {    MyDefaultMutableTreeNode nodoh=(MyDefaultMutableTreeNode)nodop.getChildAt(0);            
           isSnowFlake = !nodoh.getNameColumn().substring(0,nodoh.getNameColumn().indexOf(".")).equalsIgnoreCase(hierarchyTableName);
           nodop=nodoh;
       }
        return isSnowFlake;
    }
 private Vector getJoins(Object object) {
       Vector joinTable=new Vector();         
    for(int i = 0; i < joins.size(); i++)
        {   Conexion con1=(Conexion)joins.elementAt(i);              
            if(con1.getFromTable().equals(object.toString())|| con1.getToTable().equals(object.toString()))
            {       joinTable.add(con1);
                    
            }
        }
       return joinTable;
    }
 
private Vector getHierarchyTableJoins(MyDefaultMutableTreeNode hier)
    {    levelTables=new Vector();
         boolean esta=false;        
        MyDefaultMutableTreeNode nodop= new MyDefaultMutableTreeNode();
        nodop=hier; 
        levelTables.add(hier.getNameColumn().substring(0,hier.getNameColumn().indexOf(".")));        
         for(int i = 0; i < hier.getDepth(); i++)
       {    MyDefaultMutableTreeNode nodoh=(MyDefaultMutableTreeNode)nodop.getChildAt(0);
            for(int j=0;j<levelTables.size();j++)
            {
                if(levelTables.get(j).toString().equals(nodoh.getNameColumn().substring(0,nodoh.getNameColumn().indexOf("."))))
                {  esta=true;
                   break;
                }
            } 
            
            if(!esta)
                levelTables.add(nodoh.getNameColumn().substring(0,nodoh.getNameColumn().indexOf(".")));
            esta=false;
           nodop=nodoh;
       }      
        
        Vector joinTables = new Vector();
        String rigthtable = null,lefttable=null;        
        for(int i = 0; i < levelTables.size(); i++)
        {   joins1=new Vector();            
            joins1=getJoins(levelTables.get(i));                
            if(i==0)
            {         
                      for(int j = 0; j < joins1.size(); j++)
                        {   Conexion con1=(Conexion)joins1.elementAt(j);                               
                            if(con1.getFromTable().equals(factTableName))
                             {      rigthtable=con1.getToTable();
                                   lefttable=con1.getFromTable();
                                   break;
                              }
                            else
                            {   if(con1.getToTable().equals(factTableName))
                                {       lefttable=con1.getToTable();
                                        rigthtable=con1.getFromTable();
                                        break;
                                  }
                            }
                        }
                      if(rigthtable==null&&lefttable==null)
                      {     xmlerror=false;
                            System.out.println("error join fact");                                  
                      }
            }
            else{
                 for(int j = 0; j < joins1.size(); j++)
                    {   Conexion con1=(Conexion)joins1.elementAt(j);                         
                        if(con1.getFromTable().equals(rigthtable))
                        {   joinTables.add(con1.getToField());
                            joinTables.add(con1.getFromField());
                           rigthtable=con1.getToTable();
                                   lefttable=con1.getFromTable();
                            break;
                        }else
                            {   if(con1.getToTable().equals(rigthtable))
                                  {      joinTables.add(con1.getFromField());
                                        joinTables.add(con1.getToField());
                                        lefttable=con1.getToTable();
                                        rigthtable=con1.getFromTable();
                                         break;
                                  }
                                else
                                { xmlerror=false;
                                   System.out.println("error other join level "+i);
                                    break;
                                }
                    }
                 }
           }
        }
          Iterator it = joinTables.iterator();
        while( it.hasNext() )
            System.out.println( "endjoins "+it.next() );
        return joinTables;
    }
    
 private String getHierarchyPrimaryKey(String factTableName, String dimTableName)
    {
        String priKeyColName = null;
        int i = 0;
        do
        {
            if(i >= joins.size())
                break;
             Conexion con1=(Conexion)joins.elementAt(i);  
            if(con1.getFromTable().equals(dimTableName) && con1.getToTable().equals(factTableName))
            {
                priKeyColName = con1.getFromField();
                break;
            }
            if(con1.getToTable().equals(dimTableName) && con1.getFromTable().equals(factTableName))
            {
                priKeyColName = con1.getToField();
                break;
            }
            i++;
        } while(true);
        
        return priKeyColName;
    }
    
  private String getDimensionForeignKey(String factTableName, String dimTableName)
    {   this.joins=pnl_contenedor.getDND().getJoins();
        String foreignKeyColName = null;
        int i = 0;
        do
        {
            if(i >= joins.size())
                break;
            Conexion con1=(Conexion)joins.elementAt(i);            
            String joinRightColName = con1.getToField();
            String joinRightTableName = con1.getToTable();            
            String joinLeftColName = con1.getFromField();
            String joinLeftTableName = con1.getFromTable();
            if(factTableName.equalsIgnoreCase(joinRightTableName) && dimTableName.equalsIgnoreCase(joinLeftTableName))
            {
                foreignKeyColName = joinRightColName;
                break;
            }
            if(dimTableName.equalsIgnoreCase(joinRightTableName) && factTableName.equalsIgnoreCase(joinLeftTableName))
            {
                foreignKeyColName = joinLeftColName;
                break;
            }
            i++;
        } while(true);
        if(foreignKeyColName==null)
            xmlerror=false;
        return foreignKeyColName;
    }

    public Document getXmlDocument()
    {
        return xmlDoc;
    }
      public boolean isXMLOk()
    {
        return xmlerror;
    }

    

    private void createMeasures() {
        DefaultTableModel model=(DefaultTableModel) this.tbl_measures.getModel();
        int rowcount=model.getRowCount();
         measures=new MeasureType[rowcount-42];
        int index=0;
        String tblName,aux;
        int sqlDataType = 0,numcol;
        ResultSet result;
        aux=nombreTable(model.getValueAt(index,0).toString());
        tblName=nombreTable(aux); 
        try{ Statement s = connection.createStatement();
             result=s.executeQuery("select * from "+tblName);//ejecutar la consulta                
             ResultSetMetaData metaDatos = result.getMetaData();
             numcol=metaDatos.getColumnCount();
            while(rowcount>42)            
            {    for (int i = 1; i <= numcol; i++)                                         
                    if(metaDatos.getColumnLabel(i).equalsIgnoreCase(model.getValueAt(index,0).toString().substring(model.getValueAt(index,0).toString().indexOf(".") + 1)))
                        sqlDataType=metaDatos.getColumnType(i);
                MeasureType auxmeasure=new MeasureType(model.getValueAt(index,0).toString().substring(model.getValueAt(index,0).toString().indexOf(".")+1),sqlDataType, tblName);
                auxmeasure.setSummarizeBy(model.getValueAt(index,1).toString());
                auxmeasure.setFormat(model.getValueAt(index,2).toString());
                auxmeasure.setDataFieldName(model.getValueAt(index,3).toString());
                measures [index]=auxmeasure;
                index++;
                rowcount--;
            }
             result.close();
            }catch(SQLException e){                
                System.out.println("CD"+e.getMessage());
             }
        
    }
   private boolean compruebaType(MyDefaultMutableTreeNode nodoh,Element levelElement) {
        boolean flag=false;
        String tblName,field;
        int sqlDataType = 0,numcol;
        ResultSet result;
        tblName=nodoh.getNameColumn().substring(0, nodoh.getNameColumn().indexOf("."));
        field=nodoh.getNameColumn().substring(nodoh.getNameColumn().indexOf(".") + 1);        
        try{ 
             Statement s = connection.createStatement();
             result=s.executeQuery("select * from "+tblName);//ejecutar la consulta                
             ResultSetMetaData metaDatos = result.getMetaData();
             numcol=metaDatos.getColumnCount();
               for (int i = 1; i <=numcol; i++)                                         
               {    if(metaDatos.getColumnLabel(i).equalsIgnoreCase(field))
                        sqlDataType=metaDatos.getColumnType(i);                    
               }    
             result.close();
        }catch(SQLException e){
            System.out.println("CD"+e.getMessage());
         }
          switch(sqlDataType)
            {
                case -5: 
                case 5:                 
                case 4: // '\004'
                case 3: // '\003'
                case 6: // '\006'
                case 8: // '\b'
                case 2: // '\002
                    levelElement.addAttribute(createAttribute("type","Numeric"));
                    break;
              case 91:              
                    levelElement.addAttribute(createAttribute("type","Date"));
                  break;
              case 93:   
                  levelElement.addAttribute(createAttribute("type","Timestamp"));
                  break;
              case -7:   
                  levelElement.addAttribute(createAttribute("type","Boolean"));             
                  break;
            }
      
        return flag;
    }
   
    public void uploadXML(String cubeName){
        this.cubeName=cubeName;
        createMeasures();
        getXMLString();
        createCubeSchema();        
         f= new FileStorage(con, XMLFileName,cubeName,username,url);                            
        }
   public void uploadXML(String cubeName,DefaultTableModel modelaccess) {
        this.cubeName=cubeName;
        createMeasures();
        getXMLString();
        createCubeSchema();
        f= new FileStorage(con, XMLFileName,cubeName,username,url,modelaccess);                            
    }
     
    public FileStorage getFileStorage(){
        return f;
    }
    
    private String nombreTable(String table) {
        String nametable="";
        for(int i=0;i<table.length();i++)
        { if(table.charAt(i)!='.')
                nametable=nametable+table.charAt(i);
          else return nametable;
        }
        return nametable;
    }
    Document xmlDoc;
    Document xmlXActionDoc;
    static final String STANDARD_FORMAT = "Standard";
    static String factTableName = "";

}
