package Core.main;

import Core.queryMDX.*;
import Core.member.InterfaceMember;
import Core.operations.Dice;
import Core.tree.TreeMain;
import java.util.logging.Logger;
import java.util.*;
import java.util.Vector;
import org.olap4j.*;
import org.olap4j.metadata.*;
import Core.queryMDX.ExecuteQuery; 
import Core.queryMDX.Query; 


/**
 *
 * @author Ricardo
 */
public class Core {
    /**Esta es la clase principal
    *Es el nucleo de toda la herramienta (Manejador de cubos), contiene la consulta MDX
    *Y se encarga de administrar la interactividad entre las diferentes clases
    */ 
    public Rows rows;
    public Cols cols;
    public OlapConnection connection;
    public CellSet cellset;
    public TreeMain treeRow;
    public TreeMain treeCol;
    public Dice dice;
    boolean bandera=true;
    boolean exito=true;
    String cubeName;
    Member[] membersMeasures;
    Vector backUpCols,backUpRows;
    
    //inicialización del Core
    public Core(Vector row, Vector columns,OlapConnection connection, String cubeName){
       //vectorMember contiene en cada elemento del vector la lista de miembros de cada eje que el usuario escogio
       //es decir el vector será de 2 elementos, cada elemeno un subVectores de Miembros 
       //el primer elemento (0) pertenece a rows y el segundo (1) a cols   
      
      this.connection=connection;
      this.cubeName=cubeName;
      this.rows=new Rows(createMembers(row),createHierarchies(row));       
      this.cols=new Cols(createMembers(columns),createHierarchies(columns));
      this.dice=null;
      Query query=new Query(rows.getQuexyAxis(), cols.getQuexyAxis(),dice, this.cubeName);
      //System.out.println( cols.getQuexyAxis()+"      "+rows.getQuexyAxis());
      ExecuteQuery executeQuery=new ExecuteQuery(query.getQuery(), connection);
      exito=executeQuery.getExito();
      if(exito)
      {   this.cellset=executeQuery.getCellSet();
          this.treeCol=cols.getTreeMain();
          this.treeRow=rows.getTreeMain();     
      }
   }
    public Vector createMembers(Vector members){
      //obtiene el primer miembro de cad una de las jerarquias y las adiciona a un objeto Vector
      //que en cada indice tiene un vector de tipo Member de 1 elemento cada uno
      //y ese elemento es el primer miembro de cada jerarquia
        Vector miembros=new Vector();
        for(int i=0;i<members.size();i++){
            try {
                //esto se hace con el fin de llenar la tabla con todas las measures, si es que la dimension measure esta presente en la tabla    
                this.membersMeasures=new Member[this.connection.getSchema().getCubes().get(cubeName).getDimensions().get(0).getHierarchies().get(0)
                            .getLevels().get(0).getMembers().size()];
                    for(int contador=0;contador<this.membersMeasures.length;contador++)
                             this.membersMeasures[contador]=this.connection.getSchema().getCubes().get(cubeName).getDimensions().get(0).getHierarchies().get(0)
                            .getLevels().get(0).getMembers().get(contador);
                                    
                    Hierarchy hierarchy = (Hierarchy) members.get(i);
                    NamedList<Level> levelList = hierarchy.getLevels();
                    Level level = levelList.get(0);
                    List<Member> memberList = level.getMembers();
                    Member member = memberList.get(0);
                    Member[] memberVector = new Member[1];
                    memberVector[0] = member;
                    miembros.add(memberVector);
            } catch (OlapException ex) {
                Logger.getLogger(Core.class.getName()).log(java.util.logging.Level.SEVERE, null, ex);
            }
        }
        return miembros;
  }
    public Vector createHierarchies(Vector hierarchies){
      //Obtiene cada un d las hierarchies y las adiciona a un vector de tipo Vector      
      Vector hierarchy=new Vector();
        for(int i=0;i<hierarchies.size();i++){          
                Hierarchy hie=(Hierarchy) hierarchies.get(i);
                hierarchy.add(hie);         
        }        
      return hierarchy;
  }       
    
    //Metodos para el manejo de operaciones basicas
    public void drillDown_RollUp(int axis,int dimension,int member){
    this.backUp(0);
    this.bandera=true;
    if(axis==0)
    {    
        this.rows.DrillDown_RollUp(dimension, member);
        this.bandera=this.rows.getBandera();
    }
    else
    {
        this.cols.DrillDown_RollUp(dimension, member);
        this.bandera=this.cols.getBandera();
    }
    if(this.bandera)
    {    
        Query query=null;
        query=new Query(rows.getQuexyAxis(), cols.getQuexyAxis(),dice, this.cubeName);
        ExecuteQuery executeQuery=new ExecuteQuery(query.getQuery(), this.connection);  
        exito=executeQuery.getExito();
        if(exito)            
        {   
            this.cellset=executeQuery.getCellSet();
            this.treeCol=cols.getTreeMain();
            this.treeRow=rows.getTreeMain();      
        }
        else
            this.backUp(1);
    }
  }  
    public void pivot(){
    this.backUp(0);
    Query query=null;
    TreeMain treeAux=cols.getTreeMain();
    cols.setTreeMain(rows.getTreeMain());
    rows.setTreeMain(treeAux);
    query=new Query(rows.getQuexyAxis(), cols.getQuexyAxis(),dice, this.cubeName);
    ExecuteQuery executeQuery=new ExecuteQuery(query.getQuery(), this.connection); 
    exito=executeQuery.getExito();
    if(exito)
    {  this.cellset=executeQuery.getCellSet();
        this.treeCol=cols.getTreeMain();
        this.treeRow=rows.getTreeMain();    
    }
    else
        this.backUp(1);
  }
    public void dice(Vector elementosDice){
        this.backUp(0);
        Query query=null;
        Vector vectorAuxiliar;
        int aux=0;
        Member measureASalvar=null;
        try {        
        if(this.dice!=null){
            vectorAuxiliar=this.dice.getElementosDice();
            for(int j=0;j<vectorAuxiliar.size();j++){
                    if (((Member) vectorAuxiliar.get(j)).getDimension().getDimensionType().equals(Dimension.Type.MEASURE)) {
                        measureASalvar=(Member) vectorAuxiliar.get(j);
                    }
                    break;
            }
            this.dice=new Dice(elementosDice);
            this.dice.setElementoDice(measureASalvar);            
        }
        else{
            this.dice=new Dice(elementosDice);
        }
        query=new Query(rows.getQuexyAxis(), cols.getQuexyAxis(),dice, this.cubeName);
        ExecuteQuery executeQuery=new ExecuteQuery(query.getQuery(), this.connection);  
        exito=executeQuery.getExito();
        if(exito)
        {   this.cellset=executeQuery.getCellSet();
            this.treeCol=cols.getTreeMain();
            this.treeRow=rows.getTreeMain();            
        }
        else
            this.backUp(1);
        } catch (OlapException ex) {
            Logger.getLogger(Core.class.getName()).log(java.util.logging.Level.SEVERE, null, ex);
        }        
    }
    public void deleteMember(int axis,int dimension,int member) {
        this.backUp(0);
        this.bandera=true;
        if(axis==0)
        {    
            this.rows.deleteMember(dimension, member);
            this.bandera=this.rows.getBandera();
        }
        else
        {
            this.cols.deleteMember(dimension, member);
            this.bandera=this.cols.getBandera();
        }
        if(this.bandera)
        {    
            Query query=null;
            query=new Query(rows.getQuexyAxis(), cols.getQuexyAxis(),dice, this.cubeName);
            ExecuteQuery executeQuery=new ExecuteQuery(query.getQuery(), this.connection);  
            exito=executeQuery.getExito();
            if(exito)
            {    this.cellset=executeQuery.getCellSet();
                this.treeCol=cols.getTreeMain();
                this.treeRow=rows.getTreeMain();      
            }
            else
                this.backUp(1);
        }        
    }    
    public void backUp(int opcion){
        if(opcion==0){
            this.backUpCols=(Vector)this.cols.getTreeMain().getVectorMain().clone();
            this.backUpRows=(Vector)this.rows.getTreeMain().getVectorMain().clone();        
        }
        else{
            this.cols.setVectorMain(this.backUpCols);
            this.rows.setVectorMain(this.backUpRows);
        }

    }
    
    //Los siguientes metodos son para el manejo de las measures:
    public int findWhereMeasure(){
        int retorno=0;
        //Metodo encargado de buscar si la measure se encuentra en la tabla o en el dice,
        //si esta en la tabla en rows devuelve 1, si esta en la tabla en cols devuelve un 2
        //si esta en el dice devuelve 0, si no esta ni en la tabla ni en el dice, toma la measure por defecto y la adiciona al dice
        //y devulve un 0, en caso de que el dice no se haya creado, se crea uno nuevo con la measure por defecto y se devuelve un 0
            try {
                Vector vectorRowsHierarchies =this.rows.getVectorHierarchies();
                for(int i=0;i<vectorRowsHierarchies.size();i++){                
                    Hierarchy hierarchy = (Hierarchy) vectorRowsHierarchies.get(i);
                    if (hierarchy.getDimension().getDimensionType().equals(Dimension.Type.MEASURE)) {
                        retorno=1;
                        break;
                    }
                }
                if(retorno==0){
                    Vector vectorColsHierarchies =this.cols.getVectorHierarchies();
                    for(int i=0;i<vectorColsHierarchies.size();i++){                
                        Hierarchy hierarchy = (Hierarchy) vectorColsHierarchies.get(i);
                        if (hierarchy.getDimension().getDimensionType().equals(Dimension.Type.MEASURE)) {
                            retorno=2;
                            break;
                        }
                    }
                }                
            }catch (OlapException ex) {
                Logger.getLogger(Core.class.getName()).log(java.util.logging.Level.SEVERE, null, ex);            
            }
        if(retorno==0){
            if(dice==null){
                try {
                    Vector vectorDice = new Vector();
                    vectorDice.add(this.connection.getSchema().getCubes().get(cubeName).getDimensions().get(0).getHierarchies().get(0)
                            .getLevels().get(0).getMembers().get(0));
                            //.get(0).getUniqueName());
                    dice = new Dice(vectorDice);
                } catch (OlapException ex) {
                    Logger.getLogger(Core.class.getName()).log(java.util.logging.Level.SEVERE, null, ex);
                }
            }
        }                        
        System.out.println("La measure se encuentra en: "+retorno);
        return retorno;                
    }    
    public Vector findWhatmeasure(int opcion){
        //si opcion es 0 tengo que ir a buscar a el dice la measure, si es 1 a las rows y si es 2 a las cols
        Vector vectorRetorno=null;
        if(opcion==0){
            //Vayase a buscar a dice
            
                try {
                    for(int i=0;i<this.dice.getElementosDice().size();i++){
                        if (((Member) this.dice.getElementosDice().get(i)).getDimension().getDimensionType().equals(Dimension.Type.MEASURE)) {
                            vectorRetorno=new Vector();
                            vectorRetorno.add((Member) this.dice.getElementosDice().get(i));
                            break;
                        }
                    }
                } catch (OlapException ex) {
                    Logger.getLogger(Core.class.getName()).log(java.util.logging.Level.SEVERE, null, ex);
                }
        }
        else{
            if(opcion==1){
                //vayase a buscar a rows
                try {      
                    int valorAux=0;
                    InterfaceMember interfaceMember=new InterfaceMember(this.rows.getTreeMain().getVectorMain());
                    for(int i=0;i<interfaceMember.getNumDimension();i++){
                        for(int j=0;j<interfaceMember.getNumMember(i);j++){
                                if (interfaceMember.getMyNodeMember(i, j).getMember().getDimension().getDimensionType().equals(Dimension.Type.MEASURE)) {
                                    if(vectorRetorno==null){
                                        vectorRetorno=new Vector();
                                        vectorRetorno.add(interfaceMember.getMyNodeMember(i, j).getMember());
                                    }    
                                    else{
                                        valorAux=0;
                                        for(int k=0;k<vectorRetorno.size();k++){
                                            if(interfaceMember.getMyNodeMember(i, j).getMember().equals((Member) vectorRetorno.get(k))){
                                                valorAux=1;
                                            }
                                        }
                                        if(valorAux==0){
                                            vectorRetorno.add(interfaceMember.getMyNodeMember(i, j).getMember());
                                        }
                                    }
                                }
                        }
                    }
                } catch (OlapException ex) {
                    Logger.getLogger(Core.class.getName()).log(java.util.logging.Level.SEVERE, null, ex);
                }                    
            }
            else{
            //osea si opcion es 2 vayase a cols
                try {      
                    int valorAux=0;
                    InterfaceMember interfaceMember=new InterfaceMember(this.cols.getTreeMain().getVectorMain());
                    for(int i=0;i<interfaceMember.getNumDimension();i++){
                        for(int j=0;j<interfaceMember.getNumMember(i);j++){
                                if (interfaceMember.getMyNodeMember(i, j).getMember().getDimension().getDimensionType().equals(Dimension.Type.MEASURE)) {
                                    if(vectorRetorno==null){
                                        vectorRetorno=new Vector();
                                        vectorRetorno.add(interfaceMember.getMyNodeMember(i, j).getMember());
                                    }    
                                    else{
                                        valorAux=0;
                                        for(int k=0;k<vectorRetorno.size();k++){
                                            if(interfaceMember.getMyNodeMember(i, j).getMember().equals((Member) vectorRetorno.get(k))){
                                                valorAux=1;
                                            }
                                        }
                                        if(valorAux==0){
                                            vectorRetorno.add(interfaceMember.getMyNodeMember(i, j).getMember());
                                        }
                                    }
                                }
                        }
                    }
                } catch (OlapException ex) {
                    Logger.getLogger(Core.class.getName()).log(java.util.logging.Level.SEVERE, null, ex);
                }                
            }
        }
        return vectorRetorno;
    }
    public void alterMeasureDice(Member newMeasure){
        this.backUp(0);
        Query query=null;
        try {
            for(int i=0;i<this.dice.getElementosDice().size();i++){
                if (((Member) this.dice.getElementosDice().get(i)).getDimension().getDimensionType().equals(Dimension.Type.MEASURE)) {
                        this.dice.elementosDice.remove(i);
                    break;
                }
            }
            this.dice.setElementoDice(newMeasure);
            query=new Query(rows.getQuexyAxis(), cols.getQuexyAxis(),dice, this.cubeName);
            ExecuteQuery executeQuery=new ExecuteQuery(query.getQuery(), this.connection);  
            exito=executeQuery.getExito();
            if(exito)
            {   this.cellset=executeQuery.getCellSet();
                this.treeCol=cols.getTreeMain();
                this.treeRow=rows.getTreeMain();            
            }
            else
                this.backUp(1);
        } catch (OlapException ex) {
            Logger.getLogger(Core.class.getName()).log(java.util.logging.Level.SEVERE, null, ex);
        }    
    }
    public void deleteMeasureTable(Member measure){
        this.backUp(0);
        if(this.findWhereMeasure()==1)
        {    
            this.rows.deleteMeasure(measure);
        }
        else
        {
            this.cols.deleteMeasure(measure);
        }    
            Query query=null;
            query=new Query(rows.getQuexyAxis(), cols.getQuexyAxis(),dice, this.cubeName);
            ExecuteQuery executeQuery=new ExecuteQuery(query.getQuery(), this.connection);  
            exito=executeQuery.getExito();
            if(exito)
            {   this.cellset=executeQuery.getCellSet();
                this.treeCol=cols.getTreeMain();
                this.treeRow=rows.getTreeMain();      
            }   
            else
                this.backUp(1);
    }
    public void addMeasureTable(Member measure){
    //this.backUp(0);
    if(this.findWhereMeasure()==1)
    {    
        this.rows.addMeasure(measure);
    }
    else
    {
        this.cols.addMeasure(measure);        
    }    
        Query query=null;
        query=new Query(rows.getQuexyAxis(), cols.getQuexyAxis(),dice, this.cubeName);
        ExecuteQuery executeQuery=new ExecuteQuery(query.getQuery(), this.connection);  
        exito=executeQuery.getExito();
        if(exito)
        {   this.cellset=executeQuery.getCellSet();
            this.treeCol=cols.getTreeMain();
            this.treeRow=rows.getTreeMain();      
        }
        //else
           // this.backUp(1);
    }
    public void addAllMeasureTable(){
        for(int i=1;i<this.membersMeasures.length;i++){
            addMeasureTable(membersMeasures[i]);
        }      
    }
    
    //gets
    public Vector getVectorMainCol(){
        return this.treeCol.getVectorMain();
    }
    public Vector getVectorMainRow(){
        return this.treeRow.getVectorMain();
    }
    public CellSet getCellSet(){
            return this.cellset;
    }
    public TreeMain getTreeRow(){
            return this.treeRow;
    }
    public TreeMain getTreeCol(){
            return this.treeCol;
    }    
    public boolean getBandera(){
        return this.bandera;
    }    
    public Vector getVectorHierarchiesCols(){
        return this.treeCol.getVectorHierarchy();
    }
    public Vector getVectorHierarchiesRows(){
        return this.treeRow.getVectorHierarchy();
    }    
    public boolean getExito(){
        return this.exito;
    }    
    
}