package misc.swing.border;



import java.awt.Color;
import java.awt.Font;

import javax.swing.BorderFactory;
import javax.swing.border.BevelBorder;
import javax.swing.border.Border;
import javax.swing.border.EmptyBorder;
import javax.swing.border.EtchedBorder;
import javax.swing.border.LineBorder;
import javax.swing.border.TitledBorder;

/**
*
* @author  f00992x
*/
public class BorderBuider {
	private BorderInfo borderInfo;
	private BordersInfo bordersInfo;

	private BorderBuider(BorderInfo borderInfo) {
		this.borderInfo = borderInfo;
	}
	private BorderBuider(BordersInfo bordersInfo) {
		this.bordersInfo = bordersInfo;
	}
	public static final Border build(BorderInfo borderInfo){
		return new BorderBuider(borderInfo).buid();
	}	
	public static final Border build(BordersInfo bordersInfo){
		return new BorderBuider(bordersInfo).buidAll();
	}
	private Border buidAll() {
		Border border=null;
		Border borderPrev=null;
		int counter=bordersInfo.size();
		for(int i=counter-1; i>-1; i--){
			BorderInfo bInfo=bordersInfo.get(i);
			if(border==null){
				border=build(bInfo);
			}else{
				borderPrev=build(bInfo);
			}
			if(borderPrev!=null){
				border = BorderFactory.createCompoundBorder(border,borderPrev);
				borderPrev=null;
			}
		}
		return border;
	}
	private Border buid() {
		Border border=null;
		if(BorderInfo.EMPTY_BORDER.equals(borderInfo.getName())){
			int top=borderInfo.getIntValue(BorderInfo.KEY_TOP);
			int left=borderInfo.getIntValue(BorderInfo.KEY_LEFT);
			int bottom=borderInfo.getIntValue(BorderInfo.KEY_BOTTOM);
			int right=borderInfo.getIntValue(BorderInfo.KEY_RIGHT);
			border=new EmptyBorder( top, left, bottom, right);
		}else if(BorderInfo.LINE_BORDER.equals(borderInfo.getName())){
			Color color=(Color) borderInfo.get(BorderInfo.KEY_LINE_COLOR);
			int thickness=borderInfo.getIntValue(BorderInfo.KEY_THICKNESS);
			boolean roundedCorners=borderInfo.getBooleanValue(BorderInfo.KEY_ROUNDED_CORNERS);
			border= new LineBorder(color, thickness, roundedCorners);
		}else if(BorderInfo.ETCHED_BORDER.equals(borderInfo.getName())){
			int etchType= borderInfo.getIntValue(BorderInfo.KEY_ETCH_TYPE);
			Color highlight=(Color) borderInfo.get(BorderInfo.KEY_HIGHLIGHT);
			Color shadow=(Color) borderInfo.get(BorderInfo.KEY_SHADOW);
			border= new EtchedBorder(etchType, highlight, shadow) ;
		}else if(BorderInfo.BEVEL_BORDER.equals(borderInfo.getName())){
			int bevelType= borderInfo.getIntValue(BorderInfo.KEY_BEVEL_TYPE);
			Color highlightOuterColor=(Color) borderInfo.get(BorderInfo.KEY_HIGHLIGHT_OUTER_COLOR);
            Color highlightInnerColor=(Color) borderInfo.get(BorderInfo.KEY_HIGHLIGHT_INNER_COLOR);
            Color shadowOuterColor =(Color) borderInfo.get(BorderInfo.KEY_SHADOW_OUTER_COLOR);
            Color shadowInnerColor=(Color) borderInfo.get(BorderInfo.KEY_SHADOW_INNER_COLOR);
			border= new BevelBorder(bevelType, highlightOuterColor, 
                    highlightInnerColor, shadowOuterColor, shadowInnerColor);
		}else if(BorderInfo.TITLED_BORDER.equals(borderInfo.getName())){
			Border borderInternal=null;
			BorderInfo internalBorderInfo=borderInfo.getInternalBorderInfo();
			if(internalBorderInfo!=null 
					&& !BorderInfo.TITLED_BORDER.equals(internalBorderInfo.getName())){
				borderInternal=build(internalBorderInfo);
			}
            String title=(String) borderInfo.get(BorderInfo.KEY_TITLE);
            int titleJustification = borderInfo.getIntValue(BorderInfo.KEY_TITLE_JUSTIFICATION);
            int titlePosition = borderInfo.getIntValue(BorderInfo.KEY_TITLE_POSITION);
            Font titleFont =(Font) borderInfo.get(BorderInfo.KEY_TITLE_FONT);
            Color titleColor=(Color) borderInfo.get(BorderInfo.KEY_TITLE_COLOR);
			border= new TitledBorder(borderInternal, title, titleJustification,
                    titlePosition, titleFont, titleColor); 
		}
		return border;
	}
}
