package misc.swing.border;



import java.awt.Color;
import java.util.Collection;
import java.util.HashMap;
import java.util.Map;
import java.util.Set;
import java.util.Map.Entry;

import javax.swing.border.BevelBorder;
import javax.swing.border.EtchedBorder;
import javax.swing.border.TitledBorder;

/**
*
* @author  f00992x
*/
public class BorderInfo {
	/**
	 * Keys 
	 */
	public static final String KEY_TOP="Top";
	public static final String KEY_LEFT="Left";
	public static final String KEY_BOTTOM="Bottom";
	public static final String KEY_RIGHT="Right";
	public static final String KEY_LINE_COLOR="lineColor";
	public static final String KEY_THICKNESS="thickness";
	public static final String KEY_ROUNDED_CORNERS="roundedCorners"; //Boolean
	public static final String KEY_ETCH_TYPE="etchType"; //int
	public static final String KEY_HIGHLIGHT="highlight";
	public static final String KEY_SHADOW="shadow";//Color
	public static final String KEY_BEVEL_TYPE="bevelType";
	public static final String KEY_HIGHLIGHT_OUTER_COLOR="highlightOuterColor";
	public static final String KEY_HIGHLIGHT_INNER_COLOR="highlightInnerColor";
	public static final String KEY_SHADOW_OUTER_COLOR="shadowOuterColor";
	public static final String KEY_SHADOW_INNER_COLOR="shadowInnerColor";
	//public static final String KEY_BORDER="border";
	public static final String KEY_TITLE="title";
	public static final String KEY_TITLE_JUSTIFICATION="titleJustification";
	public static final String KEY_TITLE_POSITION="titlePosition";
	public static final String KEY_TITLE_FONT="titleFont";
	public static final String KEY_TITLE_COLOR="titleColor";
	/**
	 * Border names.
	 */
	public static final String TITLED_BORDER="TITLED BORDER";
	public static final String BEVEL_BORDER="BEVEL BORDER";
	public static final String ETCHED_BORDER="ETCHED BORDER";
	public static final String LINE_BORDER="LINE BORDER";
	public static final String EMPTY_BORDER="EMPTY BORDER";
	
	private static final String[] BORDER_NAMES={
		TITLED_BORDER,
		BEVEL_BORDER,
		ETCHED_BORDER,
		LINE_BORDER,
		EMPTY_BORDER
	};
	/**
	 * The name of border
	 */
	private String name;
	private BorderInfo internalBorderInfo;
	
	private Map<String, Object> properties =new HashMap<String, Object>();

	public BorderInfo(String name) {
		this.name = name;
		if(!isValidName(name)){
			throw new AssertionError("invalid border name: " + name);
		}
	}

	/**
	 * @return Returns the name.
	 */
	public String getName() {
		return name;
	}
	public static boolean isValidName(String name){
		for (int i = 0; i < BORDER_NAMES.length; i++) {
			if(BORDER_NAMES[i].equals(name)){
				return true;
			}
		}
		return false;
	}
	
	/**
	 * @return Returns the bORDER_NAMES.
	 */
	public static String[] getBORDER_NAMES() {
		return BORDER_NAMES;
	}

	/** 
	 * @see java.util.Map#clear()
	 */
	public void clear() {
		properties.clear();
	}
	public boolean containsKey(Object key) {
		return properties.containsKey(key);
	}

	/**
	 * @see java.util.Map#containsValue(java.lang.Object)
	 */
	public boolean containsValue(Object value) {
		return properties.containsValue(value);
	}

	/**
	 * @see java.util.Map#entrySet()
	 */
	public Set<Entry<String,Object>> entrySet() {
		return properties.entrySet();
	}

	/**
	 * @see java.util.Map#get(java.lang.Object)
	 */
	public Object get(String key) {
		Object value=properties.get(key);
		if(value==null){
			value=getDefaultVaule(key);
		}
		return value;
	}
	public int getIntValue(String key){
		if(get(key) instanceof Integer){
			return ((Integer)get(key)).intValue();
		}
		throw new AssertionError("invalid key: " + key);
	}
	public boolean getBooleanValue(String key){
		if(get(key) instanceof Boolean){
			return ((Boolean)get(key)).booleanValue();
		}
		throw new AssertionError("invalid key: " + key);
	}
	public static Object getDefaultVaule(String key){
		if(KEY_TOP.equals(key)){
			return new Integer(0);
		}else if(KEY_BOTTOM.equals(key)){
			return new Integer(0);
		}else if(KEY_LEFT.equals(key)){
			return new Integer(0);
		}else if(KEY_RIGHT.equals(key)){
			return new Integer(0);
		}else if(KEY_LINE_COLOR.equals(key)){
			return new Color(0,0,0);
		}else if(KEY_THICKNESS.equals(key)){
			return new Integer(1);
		}else if(KEY_ROUNDED_CORNERS.equals(key)){
			return new Boolean(false);
		}else if(KEY_ETCH_TYPE.equals(key)){
			return new Integer(EtchedBorder.RAISED);
		}else if(KEY_HIGHLIGHT.equals(key)){
			return null;
		}else if(KEY_SHADOW.equals(key)){
			return null;
		}else if(KEY_BEVEL_TYPE.equals(key)){
			return new Integer(BevelBorder.RAISED);
		}else if(KEY_HIGHLIGHT_OUTER_COLOR.equals(key)){
			return null;
		}else if(KEY_HIGHLIGHT_INNER_COLOR.equals(key)){
			return null;
		}else if(KEY_SHADOW_OUTER_COLOR.equals(key)){
			return null;
		}else if(KEY_SHADOW_INNER_COLOR.equals(key)){
			return null;
		}else if(KEY_TITLE.equals(key)){
			return null;
		}else if(KEY_TITLE_JUSTIFICATION.equals(key)){
			return new Integer(TitledBorder.DEFAULT_JUSTIFICATION);
		}else if(KEY_TITLE_POSITION.equals(key)){
			return new Integer(TitledBorder.DEFAULT_POSITION);
		}else if(KEY_TITLE_FONT.equals(key)){
			return null;
		}else if(KEY_TITLE_COLOR.equals(key)){
			return new Color(49,106,196);
		}
		throw new AssertionError("invalid key: " + key);
	}
	/**
	 * @see java.util.Map#isEmpty()
	 */
	public boolean isEmpty() {
		return properties.isEmpty();
	}

	public Set<String> keySet() {
		return properties.keySet();
	}

	public Object put(String key, Object value) {		
		return properties.put(key, value);
	}

	public Object remove(String key) {
		return properties.remove(key);
	}

	/**
	 * @see java.util.Map#size()
	 */
	public int size() {
		return properties.size();
	}

	/**
	 * @see java.util.Map#values()
	 */
	public Collection<Object> values() {
		return properties.values();
	}

	/**
	 * @return Returns the internalBorderInfo.
	 */
	public BorderInfo getInternalBorderInfo() {
		return internalBorderInfo;
	}

	/**
	 * @param internalBorderInfo The internalBorderInfo to set.
	 */
	public void setInternalBorderInfo(BorderInfo internalBorderInfo) {
		this.internalBorderInfo = internalBorderInfo;
	}
	
}
