package misc.swing.border;



import java.awt.Color;
import java.util.Iterator;
import java.util.Set;
import org.jdom.Attribute;
import org.jdom.Element;
import misc.xml.ColorXML;

public class BorderInfoXML {
	public static final String KEY_BORDER_INFO="BorderInfo";
	private static final String KEY_BORDER_NAME="BorderName";
	private static final String KEY_PROPERTY="Property";
	private static final String KEY_PROPERTY_NAME="NameProperty";
	private static final String KEY_PROPERTY_UNKNOWN="Unknown";
	private static final String KEY_String="String";
	private static final String KEY_Integer="Integer";
	
	private BorderInfoXML() {
		super();
	}
	public static final Element build(BorderInfo borderInfo){
		return new BorderInfoXML().getElement(borderInfo);
	}
	public static final BorderInfo build(Element eBorde){
		return new BorderInfoXML().getBorderInfo(eBorde);
	}
	private BorderInfo getBorderInfo(Element eBorde) {
		BorderInfo borderInfo=null;
		if(eBorde!=null && KEY_BORDER_INFO.equals(eBorde.getName())){
			Attribute aName=eBorde.getAttribute(KEY_BORDER_NAME);
			if(aName!=null){
				try {
					borderInfo=new BorderInfo(aName.getValue().trim());
				} catch (RuntimeException e) {
					return null;
				}
				Iterator<?> it=eBorde.getChildren(KEY_PROPERTY).iterator();
				while(it.hasNext()){
					Element e= (Element)it.next();
					PropertyInfo pInfo=getPropertyValue(e);
					if(pInfo!=null && pInfo.name!=null && pInfo.value!=null){
						borderInfo.put(pInfo.name, pInfo.value);
					}
				}
				Element eBorderInternal=eBorde.getChild(KEY_BORDER_INFO);
				if(borderInfo!=null && eBorderInternal!=null){
					borderInfo.setInternalBorderInfo(build(eBorderInternal));
				}
			}
		}
		return borderInfo;
	}
	private PropertyInfo getPropertyValue(Element element) {
		if(KEY_PROPERTY.equals(element.getName())){
			Attribute aName=element.getAttribute(KEY_PROPERTY_NAME);
			String propertyName=null;
			if(aName!=null){
				propertyName=aName.getValue().trim();
			}
			Iterator<?> it=element.getChildren().iterator();
			if(propertyName!=null && it.hasNext()){//only one child
				Element eChild=(Element) it.next();
				String text=eChild.getTextTrim();
				if(KEY_String.equalsIgnoreCase(eChild.getName())){
					return new PropertyInfo(propertyName, text);
				}else if(KEY_Integer.equalsIgnoreCase(eChild.getName())){
					try {
						int value=Integer.parseInt(text);
						return new PropertyInfo(propertyName, new Integer(value));
					} catch (NumberFormatException e) {
					}
				}else if(ColorXML.KEY_COLOR.equalsIgnoreCase(eChild.getName())){
					Color color=ColorXML.build(eChild);
					if(color!=null){
						return new PropertyInfo(propertyName, color);
					}
				}
			}
		}
		return null;
	}
	///////////////////////////////////////
	private Element getElement(BorderInfo borderInfo) {
		Element eBorde= new Element(KEY_BORDER_INFO);
		Attribute aName=new Attribute(KEY_BORDER_NAME, borderInfo.getName());
		eBorde.setAttribute(aName);
		
		Set<String> set=borderInfo.keySet();
		String[] keys= set.toArray(new String[set.size()]);
		for (int i = 0; i < keys.length; i++) {
			Object value=borderInfo.get(keys[i]);
			if(value!=null){
				Element eProperty= new Element(KEY_PROPERTY);
				eProperty.addContent(getPropertyXML(eProperty, value, keys[i]));
				eBorde.addContent(eProperty);
			}
		}
		BorderInfo borderInfoInternal=borderInfo.getInternalBorderInfo();
		if(borderInfoInternal!=null){
			Element eBInternal=build(borderInfoInternal);
			if(eBInternal!=null){
				eBorde.addContent(eBInternal);
			}
		}
		return eBorde;
	}
	private Element getPropertyXML(Element eParent, Object value, String name){
		Element element=new Element(KEY_PROPERTY_UNKNOWN);
		Attribute aName=new Attribute(KEY_PROPERTY_NAME, name);
		eParent.setAttribute(aName);
		if(value instanceof String){
			element=new Element(KEY_String);
			element.setText((String)value);
		}else if(value instanceof Integer){
			element=new Element(KEY_Integer);
			element.setText(((Integer)value).toString());			
		}else if(value instanceof Color){
			element=ColorXML.getRootElement((Color)value);
		}
		return element;
	}
	
	private class PropertyInfo{
		String name;
		Object value;
		private PropertyInfo(String name, Object value) {
			super();
			this.name = name;
			this.value = value;
		}		
	}
	
}
