package misc.swing.comboColor;



import java.awt.Color;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.SortedMap;

import javax.swing.JColorChooser;
import javax.swing.JComboBox;

public class ComboHandle {
	private JComboBox combo;
	private SortedMap<String, Color> map;
	private ActionListener actionListener;
	private Color selectedColor;
	private SelectedColorChangeSupport selectedColorChangeSupport;
	private boolean styleChanging;

	
	/**
	 * @param model
	 */
	ComboHandle(JComboBox combo, SortedMap<String, Color> map) {
		this.combo = combo;
		this.map=map;
		selectedColorChangeSupport=new SelectedColorChangeSupport();
	}


	/**
	 * @return the combo
	 */
	public JComboBox getCombo() {
		return combo;
	}
	/**
	 * @return the selectedColor
	 */
	public Color getSelectedColor() {
		return selectedColor;
	}


	/**
	 * @param selectedColor the selectedColor to set
	 */
	public void setSelectedColor(Color selectedColor) {
		if(this.selectedColor!=null 
				&& this.selectedColor.equals(selectedColor)){
			return;
		}
		styleChanging=true;
		this.selectedColor = selectedColor;
		if(selectedColor!=null){
			String[] keys=map.keySet().toArray(new String[map.keySet().size()]);
			for(int i=0; i<keys.length; i++){
				String key=keys[i];
				Color c=map.get(key);
				if(c!=null){
					if(c.equals(selectedColor)){
						combo.getModel().setSelectedItem(key);
						styleChanging=false;
						return;
					}
				}				
			}
			combo.getModel().setSelectedItem(ColorNames.OTHER);
		}
		styleChanging=false;
	}
	
	/**
	 * @return the actionListener
	 */
	ActionListener getActionListener() {
		if(actionListener==null){
			actionListener=new ActionListenerImp();
		}
		return actionListener;
	}
	public void addActionListener(ActionListener l){
		selectedColorChangeSupport.addActionListener(l);
	}
	private class ActionListenerImp implements ActionListener{
		public void actionPerformed(ActionEvent e) {
			if(!styleChanging){
				JComboBox combo=(JComboBox) e.getSource();
				String cName=(String) combo.getSelectedItem();
				Color c=map.get(cName);
				if(c!=null){
					selectedColor=c;
					selectedColorChangeSupport.fireSelectionChange(e);
				}else {				
					Color c1=JColorChooser.showDialog(combo.getParent()
							, ColorNames.OTHER, selectedColor);
					if(c1!=null){
						selectedColor=c1;
						selectedColorChangeSupport.fireSelectionChange(e);
					}
				}
			}
		}		
	}

	
}
