package misc.swing.degraded;

import java.awt.Color;

import javax.swing.BorderFactory;
import javax.swing.Icon;
import javax.swing.JLabel;
import javax.swing.SwingConstants;
import javax.swing.plaf.LabelUI;

public class DegradedLabel extends JLabel {
	private static final long serialVersionUID = 1L;
	
	protected static final Color DEFAULT_MAIN_COLOR=Color.blue;
	private static final Color NONE = new Color(0, 0, 0, 0);
	
	protected Color mainColor=DEFAULT_MAIN_COLOR;
	protected Color degradedColor=NONE;
	protected boolean degraded=true;
	private int degradedOrientation=SwingConstants.EAST;
	
	public DegradedLabel(){	
		super();
		setBorder(BorderFactory.createEmptyBorder());
	}
	public DegradedLabel(Color color){
		this();
		mainColor = color;
	}
	public DegradedLabel(String text){
		this();
		setText(text);
	}
	public DegradedLabel(Icon icon){
		this();
		setIcon(icon);
	}
	public DegradedLabel(String text, Icon icon){
		this();
		setText(text);
		setIcon(icon);
	}
    public void updateUI() {
        setUI(new DegradedLabelUI());
    }
	
    public void setUI(LabelUI ui) {
    	if(ui instanceof DegradedLabelUI){
    		super.setUI(ui);
    	}
    }
	
	public void setTransparent(){
		degradedColor=NONE;
		repaint();
	}
	public boolean isTransparent(){
		return  NONE.equals(degradedColor);
	}
	/**
	 * @return Returns the degradedColor.
	 */
	public Color getDegradedColor() {
		return degradedColor;
	}

	/**
	 * @param degradedColor The degradedColor to set.
	 */
	public void setDegradedColor(Color degradedColor) {
		if(degradedColor!=null){
			if(!degradedColor.equals(this.degradedColor)){
				this.degradedColor = degradedColor;
				repaint();
			}
		}else{
			throw new IllegalArgumentException(); 
		}
	}
	/**
	 * @return Returns the mainColor.
	 */
	public Color getMainColor() {
		return mainColor;
	}
	
	/**
	 * @param mainColor The mainColor to set.
	 */
	public void setMainColor(Color mainColor) {
		if(mainColor!=null){
			if(!mainColor.equals(this.mainColor)){
				this.mainColor = mainColor;
				repaint();
			}
		}else{
			throw new IllegalArgumentException();
		}
	}
	/**
	 * @return Returns the degraded.
	 */
	public boolean isDegraded() {
		return degraded;
	}
	/**
	 * @param degraded The degraded to set.
	 */
	public void setDegraded(boolean degraded) {
		this.degraded = degraded;
//		boolean repaint=this.degraded != degraded;
//		if(repaint){
//			repaint();
//		}
	}
	/**
	 * @return Returns the degradedOrientation.
	 */
	public int getDegradedOrientation() {
		return degradedOrientation;
	}
	/**
	 * @param degradedOrientation The degradedOrientation to set.
	 *	SwingConstants.EAST:
	 *	SwingConstants.WEST:
	 *	SwingConstants.NORTH:
	 *	SwingConstants.NORTH:
	 *	SwingConstants.SOUTH:
	 */
	public void setDegradedOrientation(int degradedOrientation) {
		switch (degradedOrientation) {
		case SwingConstants.EAST:
		case SwingConstants.WEST:
		case SwingConstants.NORTH:
		case SwingConstants.SOUTH:
			this.degradedOrientation = degradedOrientation;
			break;
		default:
			throw new IllegalArgumentException();
		}
	}	
}
