package misc.swing.degraded;

import java.awt.Color;
import java.awt.GradientPaint;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.LayoutManager;
import java.awt.Paint;

import javax.swing.JPanel;
import javax.swing.SwingConstants;

public class DegradedPanel extends JPanel {
	private static final long serialVersionUID = 1L;
	
	protected static final Color DEFAULT_MAIN_COLOR=Color.blue;
	private static final Color NONE = new Color(0, 0, 0, 0);
	
	protected Color mainColor=DEFAULT_MAIN_COLOR;
	protected Color degradedColor=NONE;
	protected boolean degraded=true;
	private int degradedOrientation=SwingConstants.EAST;
	
	public DegradedPanel(){	
		super();
		setOpaque(false);
	}
	public DegradedPanel(Color color){
		this();
		mainColor = color;
	}
	public DegradedPanel(LayoutManager layout){
		this();
		setLayout(layout);
	}
	//TODO  crear la clase class DegradedPanelUI extends BasicPanelUI
	// ver DegradedLabel
	protected void paintComponent(Graphics g){
		Graphics2D g2 = (Graphics2D) g;
		Paint oldPaint=g2.getPaint();
		if(degraded){
			GradientPaint paint;
			switch (degradedOrientation) {
				case SwingConstants.NORTH:
					paint = new GradientPaint(0, 0, degradedColor
								, 0, getHeight(), mainColor);
					break;
				case SwingConstants.EAST:
					paint = new GradientPaint(0, 0, mainColor
							, getWidth(), 0, degradedColor);
					break;
				case SwingConstants.SOUTH:
					paint = new GradientPaint(0, getHeight(), degradedColor
							, 0, 0, mainColor);
					break;
				case SwingConstants.WEST:
					paint = new GradientPaint(getWidth(), 0, mainColor
							, 0, 0, degradedColor);
					break;
				default:
					throw new UnsupportedOperationException();
			}
			g2.setPaint(paint);			
			g2.fill(getBounds());// mejor usar g2.fillRect
		}
		super.paintComponent(g);
		g2.setPaint(oldPaint);
	}
	public void setTransparent(){
		degradedColor=NONE;
		repaint();
	}
	public boolean isTransparent(){
		return  NONE.equals(degradedColor);
	}
	/**
	 * @return Returns the degradedColor.
	 */
	public Color getDegradedColor() {
		return degradedColor;
	}

	/**
	 * @param degradedColor The degradedColor to set.
	 */
	public void setDegradedColor(Color degradedColor) {
		if(degradedColor!=null){
			this.degradedColor = degradedColor;
			repaint();
		}else{
			throw new IllegalArgumentException(); 
		}
	}
	/**
	 * @return Returns the mainColor.
	 */
	public Color getMainColor() {
		return mainColor;
	}
	
	/**
	 * @param mainColor The mainColor to set.
	 */
	public void setMainColor(Color mainColor) {
		if(mainColor!=null){
			this.mainColor = mainColor;
			repaint();
		}else{
			throw new IllegalArgumentException();
		}
	}
	/**
	 * @return Returns the degraded.
	 */
	public boolean isDegraded() {
		return degraded;
	}
	/**
	 * @param degraded The degraded to set.
	 */
	public void setDegraded(boolean degraded) {
		this.degraded = degraded;
	}
	/**
	 * @return Returns the degradedOrientation.
	 */
	public int getDegradedOrientation() {
		return degradedOrientation;
	}
	/**
	 * @param degradedOrientation The degradedOrientation to set.
	 *	SwingConstants.EAST:
	 *	SwingConstants.WEST:
	 *	SwingConstants.NORTH:
	 *	SwingConstants.NORTH:
	 *	SwingConstants.SOUTH:
	 */
	public void setDegradedOrientation(int degradedOrientation) {
		switch (degradedOrientation) {
		case SwingConstants.EAST:
		case SwingConstants.WEST:
		case SwingConstants.NORTH:
		case SwingConstants.SOUTH:
			this.degradedOrientation = degradedOrientation;
			break;
		default:
			throw new IllegalArgumentException();
		}
	}
}
