package misc.swing.fontChooser;

import java.awt.Component;
import java.awt.Font;
import java.awt.HeadlessException;
import java.awt.Window;
import java.net.URL;

import javax.swing.AbstractButton;
import javax.swing.JDialog;
import javax.swing.JFrame;
import javax.swing.JList;
import javax.swing.JPanel;
import javax.swing.JTextField;
import javax.swing.ListSelectionModel;
import javax.swing.SwingUtilities;
import javax.swing.border.CompoundBorder;
import javax.swing.border.TitledBorder;

import com.jeta.forms.components.panel.FormPanel;
import misc.swing.abeille.FormLoader;
import misc.swing.border.ui.Messages;
import misc.swing.border.ui.Messages;
import misc.swing.combofont.FontNameCellRender;
import misc.swing.combofont.FontNameModel;
import misc.swing.combofont.FontSizeModel;
import misc.swing.combofont.FontStyleModel;
import misc.swing.combofont.StyleCellRender;
import misc.util.IntegerDocument;



public class FontChooser {
	private String title;
	private Font initialFont;
	
	
	private FontChooser(String title, Font initialFont) {
		super();
		this.title = title;
		this.initialFont = initialFont;
	}
	public static Font showDialog(Component component,
	        String title, Font initialFont) {
		return new FontChooser(title, initialFont).showDialog(component); 
	}
	public static Font showDialog(Component component,Font initialFont) {
		return new FontChooser(null, initialFont).showDialog(component); 
	}
//	public static JPanel showDialog(Font initialFont) {
//		return new FontChooser(null, initialFont).showDialog(); 
//	}
//	private JPanel showDialog(){
//		FormPanel panel = getFormPanel();
//		if(panel!=null){
//			FontController controller=new FontController(panel,initialFont);
//			initForm(panel, controller);
//		}
//		return panel;
//	}

	private Font showDialog(Component component){
		FormPanel panel = getFormPanel();
		if(panel!=null){
			FontController controller=new FontController(panel,initialFont);
			initForm(panel, controller);
			JDialog dialog = createDialog(component, panel);
			dialog.setIgnoreRepaint(false);
			dialog.setVisible(true);
			return controller.getSelectedFont();
		}
		return null;
	}
	private String getTitle(){
		if(title==null){
			title=Messages.getString("FontChooser.formTitled"); //$NON-NLS-1$
		}
		return title;
	}
	private JDialog createDialog(Component c, JPanel panel) throws HeadlessException {
		Window wRoot=(Window) SwingUtilities.getRootPane(c).getParent();
		
		JDialog dialog=null;
		if(wRoot instanceof JFrame)
			dialog=new JDialog((JFrame)wRoot,true);
		else
			dialog=new JDialog((JDialog)wRoot, true);
		dialog.setDefaultCloseOperation(JDialog.DISPOSE_ON_CLOSE);
		dialog.getContentPane().add(panel);
		dialog.pack();
		dialog.setLocationRelativeTo(wRoot);
		return dialog;
	}
	
	private FormPanel getFormPanel(){
		try {
			URL url=getClass().getClassLoader().getResource(FontConst.FORM_FULL_NAME);
			URL urlParent=null;
			if(url!=null){
				urlParent = new URL(url.toString().substring(0
						, url.toString().length()-FontConst.FORM_NAME.length()));
			}
			return FormLoader.load(FontConst.FORM_FULL_NAME, urlParent, getClass().getClassLoader());
		} catch (Exception e) {
			e.printStackTrace();
		}
		return null;
	}
	private void initForm(FormPanel panel, FontController controller) {
		panel.getLabel(FontConst.FORM_TITLE)
			.setText(getTitle());
		panel.getLabel(FontConst.NAME_FONT_LABEL)
			.setText(Messages.getString("FontChooser.fontName")); //$NON-NLS-1$
		panel.getLabel(FontConst.STYLE_FONT_LABEL)
			.setText(Messages.getString("FontChooser.fontStyle")); //$NON-NLS-1$
		panel.getLabel(FontConst.SIZE_FONT_LABEL)
			.setText(Messages.getString("FontChooser.fontSize")); //$NON-NLS-1$
		
		JTextField text=panel.getTextField(FontConst.SIZE_TEXT);
		text.setDocument(new IntegerDocument());
		text.getDocument().addDocumentListener(controller.getTextListener());
		
		AbstractButton b=panel.getButton(FontConst.BUTTON_OK);
		b.setText(Messages.getString("FontChooser.ok")); //$NON-NLS-1$
		b.setActionCommand(FontActionCommands.AC_BUTTON_OK);
		b.addActionListener(controller.getActionListener());
		
		b=panel.getButton(FontConst.BUTTON_CANCEL);
		b.setText(Messages.getString("FontChooser.cancel")); //$NON-NLS-1$
		b.setActionCommand(FontActionCommands.AC_BUTTON_CANCEL);
		b.addActionListener(controller.getActionListener());
		
		initSamplePanel(panel);
			
		setupListFont(panel, controller);
		setupListFontStyle(panel, controller);
		setupListFontSize(panel, controller);
		
		controller.initialize();
	}

	private void initSamplePanel(FormPanel panel){
		JPanel samplePanel=panel.getPanel(FontConst.FONT_SAMPLE_PANEL);
		CompoundBorder border=(CompoundBorder) samplePanel.getBorder();
		TitledBorder tBorder=(TitledBorder) border.getOutsideBorder();
		tBorder.setTitle(Messages.getString("FontChooser.sample")); //$NON-NLS-1$
		
		panel.getLabel(FontConst.FONT_SAMPLE_LABEL)
			.setText(Messages.getString("FontChooser.sampleText")); //$NON-NLS-1$
	}
	private void setupListFont(FormPanel panel, FontController controller){
		JList list=panel.getList(FontConst.FAMILE_LIST);
		list.setModel(new FontNameModel());
		list.setCellRenderer(new FontNameCellRender());
		list.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
		list.addListSelectionListener(controller.getSelectionListener());
	}
	private void setupListFontStyle(FormPanel panel, FontController controller) {
		JList list=panel.getList(FontConst.STYLE_LIST);
		list.setModel(new FontStyleModel());
		list.setCellRenderer(new StyleCellRender());
		list.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
		list.addListSelectionListener(controller.getSelectionListener());
	}
	private void setupListFontSize(FormPanel panel, FontController controller) {
		JList list=panel.getList(FontConst.SIZE_LIST);
		list.setModel(new FontSizeModel());
		list.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
		list.addListSelectionListener(controller.getSelectionListener());
	}
	
}
