package misc.swing.fontChooser;

import java.awt.Font;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.JDialog;
import javax.swing.JList;
import javax.swing.JTextField;
import javax.swing.ListModel;
import javax.swing.SwingUtilities;
import javax.swing.event.DocumentEvent;
import javax.swing.event.DocumentListener;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;

import com.jeta.forms.components.panel.FormPanel;
import misc.swing.border.ui.Messages;

public class FontController {
	private FormPanel panel;
	private Font initialFont;
	private Font activeFont;
	private Font selectedFont;
	private ListSelectionListener selectionListener;
	private ActionListener actionListener;
	private DocumentListener textListener;
	
	
	FontController(FormPanel panel, Font initialFont) {
		super();
		this.panel = panel;
		this.initialFont = initialFont;
		if(initialFont!=null){
			activeFont=new Font(initialFont.getName(), initialFont.getStyle(), initialFont.getSize());
		}else{
			activeFont=panel.getLabel(FontConst.FONT_SAMPLE_LABEL).getFont();
		}
	}
	private void readFont(){		
		JList list=panel.getList(FontConst.FAMILE_LIST);
		if(list.getSelectedIndex()>-1){
			Font font=(Font) list.getSelectedValue();
			//Set styleFont
			list=panel.getList(FontConst.STYLE_LIST);
			int fontStyle=font.getStyle();
			if(list.getSelectedIndex()>-1){
				fontStyle=((Integer)list.getSelectedValue());
			}
			//set sizeFont
			list=panel.getList(FontConst.SIZE_LIST);
			int fontSize=font.getSize();
			String s=panel.getTextComponent(FontConst.SIZE_TEXT).getText();
			try {
				fontSize=Integer.parseInt(s);
			} catch (NumberFormatException e1) {
			}
			activeFont=new Font(font.getName(),fontStyle,fontSize);
		}
	}
	void initialize(){
		setFont(activeFont);
	}
	private void setFont(Font font) {
		JList list=panel.getList(FontConst.FAMILE_LIST);
		ListModel model=list.getModel();
		for(int i=0; i<model.getSize(); i++){
			Font f=(Font)model.getElementAt(i);
			if(f.getName().equals(font.getName())){
				list.setSelectedValue(model.getElementAt(i), true);
				break;
			}
		}
		//Set styleFont
		list=panel.getList(FontConst.STYLE_LIST);
		model=list.getModel();
		for(int i=0; i<model.getSize(); i++){
			if(font.getStyle()==(Integer)model.getElementAt(i)){
				list.setSelectedIndex(i);
				break;
			}
		}
		//set sizeFont
		panel.getTextField(FontConst.SIZE_TEXT)
			.setText(Integer.toString(font.getSize()));
		
	}

	private void updateFontLabel(){
		if(activeFont!=null){
			panel.getLabel(FontConst.FONT_SAMPLE_LABEL).setFont(activeFont);
		}
	}
	public Font getSelectedFont(){
		return selectedFont;
	}
	/**
	 * @return the selectionListener
	 */
	ListSelectionListener getSelectionListener() {
		if(selectionListener==null){
			selectionListener=new SelectionListener();
		}
		return selectionListener;
	}
	
	/**
	 * @return the actionListener
	 */
	ActionListener getActionListener() {
		if(actionListener==null){
			actionListener= new ActionListenerImp();
		}
		return actionListener;
	}
	/**
	 * @return the textListener
	 */
	DocumentListener getTextListener() {
		if(textListener==null){
			textListener=new TextListener();
		}
		return textListener;
	}
	
	private class SelectionListener implements ListSelectionListener{

		public void valueChanged(ListSelectionEvent e) {
			if(e.getSource() instanceof JList){
				JList list=(JList) e.getSource();
				updateFontPanel(list);
			}
		}

		private void updateFontPanel(JList list) {
			String name=list.getName();
			Object value=list.getSelectedValue();
			if(FontConst.FAMILE_LIST.equals(name)){				
				if(list.getSelectedIndex()>-1){
					Font font=(Font) value;
					panel.getTextField(FontConst.NAME_FONT_TEXT)
						.setText(font.getFontName());
				}else{
					panel.getTextField(FontConst.NAME_FONT_TEXT)
						.setText(new String()); 
				}
			}else if(FontConst.STYLE_LIST.equals(name)){
				if(list.getSelectedIndex()>-1){
					panel.getTextField(FontConst.STYLE_TEXT)
						.setText( getStringValue((Integer)list.getSelectedValue()));
				}else{
					panel.getTextField(FontConst.STYLE_TEXT)
						.setText(new String());
				}
			}else if(FontConst.SIZE_LIST.equals(name)){
				if(list.getSelectedIndex()>-1){
					//String s=(String) list.getSelectedValue();
					Integer iValue=(Integer) list.getSelectedValue();
					if(!iValue.toString().equals(panel
							.getTextField(FontConst.SIZE_TEXT).getText())){
						panel.getTextField(FontConst.SIZE_TEXT)
							.setText(iValue.toString());
					}
				}
			}
			readFont();
			updateFontLabel();
		}
		private String getStringValue(int style){
			String result=null;
			switch (style) {
			case Font.PLAIN:
				result=Messages.getString("FontController.Plain"); //$NON-NLS-1$
				break;
			case Font.BOLD:
				result=Messages.getString("FontController.Bold"); //$NON-NLS-1$
				break;
			case Font.ITALIC:
				result=Messages.getString("FontController.Italic"); //$NON-NLS-1$
				break;
			case Font.BOLD|Font.ITALIC:
				result=Messages.getString("FontController.BoldItalic"); //$NON-NLS-1$
				break;

			default:
				result=Messages.getString("FontController.Unknown"); //$NON-NLS-1$
			}
			return result;
		}
	}// End class SelectionListener
	
	private class TextListener implements DocumentListener{

		public void changedUpdate(DocumentEvent e) {
			updateList();					
		}

		public void insertUpdate(DocumentEvent e) {
			updateList();					
		}

		public void removeUpdate(DocumentEvent e) {
			updateList();					
		}
		private void updateList() {
			JTextField text=panel.getTextField(FontConst.SIZE_TEXT);
			String sValue=text.getText();
			panel.getButton(FontConst.BUTTON_OK).setEnabled(sValue.trim().length()>0);
			JList list=panel.getList(FontConst.SIZE_LIST);
			Integer sSelected=(Integer) list.getSelectedValue();
			if(sSelected==null || !sValue.equals(sSelected.toString())){
				if(sSelected!=null){
					list.clearSelection();
				}
				ListModel model=list.getModel();
				for(int i=0; i<model.getSize(); i++){
					if(sValue.equals(((Integer)model.getElementAt(i)).toString())){
						list.setSelectedValue(model.getElementAt(i), true);
						break;
					}
				}
			}
			
		}

		
	}
	private class ActionListenerImp implements ActionListener{
		public void actionPerformed(ActionEvent e) {
			String ac=e.getActionCommand();
			if(FontActionCommands.AC_BUTTON_OK.equals(ac)){
				readFont();
				selectedFont=activeFont;
				closeWindow();
			}else if(FontActionCommands.AC_BUTTON_CANCEL.equals(ac)){
				setFont(initialFont);
				readFont();
				closeWindow();
			}
			
		}
		private void closeWindow(){
			JDialog dialog=(JDialog) SwingUtilities.getRootPane(panel).getParent();
			dialog.setVisible(false);
			dialog.dispose();
		}

		
	}// End class ActionListenerImp	
}

