package misc.swing.table;



import java.awt.Color;
import java.awt.Component;
import java.awt.Font;

import javax.swing.JLabel;
import javax.swing.JTable;
import javax.swing.UIManager;
import javax.swing.border.Border;
import javax.swing.border.EmptyBorder;
import javax.swing.table.TableCellRenderer;
import misc.swing.table.settings.Style;
import misc.swing.table.settings.StyleFactory;
import misc.swing.table.settings.Styles;





public class DefaultSpanCellRenderer extends JLabel implements TableCellRenderer{
	private static final long serialVersionUID = 1L;
	protected static Border noFocusBorder; 
	  
	  public DefaultSpanCellRenderer() {
	    noFocusBorder = new EmptyBorder(1, 2, 1, 2);
	    setOpaque(true);
	    setBorder(noFocusBorder);  
	  }

	  public Component getTableCellRendererComponent(JTable table, Object value,
	                 boolean isSelected, boolean hasFocus, int row, int column) {
	    Color foreground = null;
	    Color background = null;
	    Font font = null;
	    Style style=getStyle((SpanTable) table, row, column);
		if(style!=null){
		    if (isSelected) {
			      setForeground((foreground != null) ? foreground
			                          : table.getSelectionForeground());
			      setBackground(table.getSelectionBackground());
			    } else {
			      setForeground((foreground != null) ? foreground 
						  : style.getForeground());
			      setBackground((background != null) ? background 
						  : style.getBackground());
			    }
			    setFont((font != null) ? font : style.getFont());
			    setHorizontalAlignment(style.getHorizontalAlignment());			    			
		}else{
		    if (isSelected) {
			      setForeground((foreground != null) ? foreground
			                          : table.getSelectionForeground());
			      setBackground(table.getSelectionBackground());
			    } else {
			      setForeground((foreground != null) ? foreground 
						  : table.getForeground());
			      setBackground((background != null) ? background 
						  : table.getBackground());
			    }
			    setFont((font != null) ? font : table.getFont());
			    setHorizontalAlignment(((SpanTable)table).getHorizontalTextAlignment());
		}	    
	    if (hasFocus) {
	      setBorder( UIManager.getBorder("Table.focusCellHighlightBorder") );
	      if (table.isCellEditable(row, column)) {
			setForeground((foreground != null) ? foreground
			              : UIManager.getColor("Table.focusCellForeground") );
			setBackground( UIManager.getColor("Table.focusCellBackground") );
	      }
	    } else {
	      setBorder(noFocusBorder);
	    }
	    setValue(value);        
	    return this;
	  }
	    
	  protected void setValue(Object value) {
	    setText((value == null) ? "" : value.toString());
	  }
	  protected Style getStyle(SpanTable table, int row ,int column){
		  Style style=null;
		    try {
			    Span span=table.getSpanModel().getSpanAt(row,column);
			    Styles styles=table.getStyles();
			    if(styles!=null ){
			    	if(span.spanC!=1 || span.spanR!=1){
			    		style=styles.get(StyleFactory.STYLE_SPAN);
			    	}else{
			    		if(row%2==0){
			    			style=styles.get(StyleFactory.STYLE_EVEN);
			    		}else{
			    			style=styles.get(StyleFactory.STYLE_ODD);
			    		}
			    	}
			    }
			} catch (SpanException e) {
			}
		 return style; 
	  }

}
