package misc.swing.table;



import javax.swing.table.TableModel;


public class DefaultSpanModel extends AbstractSpanModel{
	protected TableModel tableModel; 
	protected Span[][] spans=new Span[0][0]; 


	/**
	 * @param tableModel
	 */
	public DefaultSpanModel(TableModel tableModel) {
		if(tableModel==null){
			throw new IllegalArgumentException("tableModel is null");
		}
		this.tableModel = tableModel;
		iniSpan();
	}
	protected void iniSpan(){
		int cols=tableModel.getColumnCount();
		int rows=tableModel.getRowCount();
		spans=new Span[rows][cols];
		for(int i=0; i< rows; i++){
			for(int j=0; j<cols; j++){
				spans[i][j]=Span.EMPTY_SPAN;
			}
		}
	}
	public int getColumnCount() {
		return tableModel.getColumnCount();
	}
	public String getColumnName(int col) {
		return tableModel.getColumnName(col);
	}
	public int getRowCount() {
		return tableModel.getRowCount();
	}
	public void setTableModel(TableModel tm){
		tableModel=tm;
		iniSpan();
		//the fire notification ??
		//fire...
	}
	
	/**
	 * @return Returns the tableModel.
	 */
	public TableModel getTableModel() {
		return tableModel;
	}
	public CellPosition getVisibleCell(int row, int col) throws SpanException{
		validateIndex(row, col);
		Span span=spans[row][col];
		if(span.isVisible()){
			return new CellPosition(row, col);
		}else{
			return new CellPosition(row+span.spanR
					, col +span.spanC);

		}		
	}
	public Span getSpanAt(int row, int col)throws SpanException {
		validateIndex(row, col);
		return spans[row][col];
	}
	public void combine(int row, int col, int rowSpan, int colSpan) throws SpanException {
		setSpan(row, col, rowSpan, colSpan);
	}
	public boolean canCombine(int row, int col, int rowSpan, int colSpan){
		try {
			canSetSpan(row, col, rowSpan, colSpan);
			return true;
		} catch (SpanException e) {
			return false;
		}
	}
	public void split(int row, int col) throws SpanException {
		CellPosition cp = getVisibleCell(row, col);
		splitCell(cp.getRow(), cp.col);
	}
	public boolean canSplit(int row, int col){
		try {
			CellPosition cp =getVisibleCell(row, col);
			return cp.row==row && cp.col==col;
		} catch (SpanException e) {
			return false;
		}
	}
	public void clear(){
		iniSpan();
	}
	public boolean isVisible(int row, int col)throws SpanException {
		validateIndex(row, col);
		Span span=spans[row][col];
		return span.isVisible();
	}
	public void validateIndex(int row, int col)throws SpanException {
		if(row< 0
			|| row>=spans.length){
			throw new SpanException("row "+row+" out rank"); 
		}
		if(col< 0
				|| col>=spans[0].length){
				throw new SpanException("col "+col+" out rank");
			}
	}
	protected void setSpan(int row, int col, int rowSpan, int colSpan) throws SpanException{
		if(rowSpan>1 ||colSpan>1){
			canSetSpan(row,col, rowSpan, colSpan);
			Span span;
			for(int i=row; i <row+rowSpan; i++){
				for(int j=col; j<col+colSpan; j++){
					if(i==row && j==col){
						span=new Span(rowSpan, colSpan);
					}else{
						span=new Span((row-i), (col-j));
					}
					spans[i][j]=span;
				}
			}
		}
		//printSpan();
	}
	protected void canSetSpan(int row, int col, int rowSpan, int colSpan) throws SpanException{
		validateIndex(row, col);
		validateIndex(row+rowSpan-1,col+colSpan-1);
		int rSpan=(row+rowSpan<spans.length)?(row+rowSpan):spans.length-1;
		int cSpan=(col+colSpan<spans[0].length)?(col+colSpan):spans[0].length-1;
		validateIndex(rSpan, cSpan);
		for(int i=row; i < rSpan; i++){
			for(int j=col; j < cSpan; j++){
				Span span=spans[i][j];
				if(span!=Span.EMPTY_SPAN){
					throw new SpanException("the position row: "
							+i+" col: "+j+" is busy."); 
				}
			}
		}
	}
	protected void splitCell(int row, int col) throws SpanException {
		Span span=getSpanAt(row, col);
		if(span.isVisible()){
			for(int i=row; i<row+span.spanR; i++){
				for(int j=col; j<col+span.spanC; j++){
					spans[i][j]=Span.EMPTY_SPAN;
				}
			}
			//printSpan();
		}else{
			throw new SpanException("row: "+row+" col:"+col+" not visible");
		}
	}
//	/**
//	 * For degug.
//	 *
//	 */
//	private void printSpan(){
//		System.out.println("----- Span ---------");
//		for(int i=0; i<spans.length;i++ ){
//			System.out.print("row: "+i+"\t");
//			for (int j = 0; j < spans[i].length; j++) {
//				System.out.print("\t"+spans[i][j].toString());				
//			}
//			System.out.println();
//		}
//		System.out.println("---- End Span ------");
//	}
		
}
