package misc.swing.table;



import javax.swing.table.TableModel;


public interface SpanModel{
    /**
     * @return the number of rows
     */
    public int getRowCount();

    /**
     * @return the number of columns
     */
    public int getColumnCount();

    /**
     *
     * @param	col	the index of the column
     * @return  the name of the column
     */
    public String getColumnName(int col);

    public TableModel getTableModel();
    
    public void setTableModel(TableModel tm);
    /**
     * Returns the span for the cell at <code>columnIndex</code> and
     * <code>rowIndex</code>.
     *
     * @param	row	the row whose value is to be queried
     * @param	col 	the column whose value is to be queried
     * @return	the value span at the specified cell
     */
    public Span getSpanAt(int row, int col) throws SpanException;

    public CellPosition getVisibleCell(int row, int col) throws SpanException;
    /**
     * Combine the cell at <code>columnIndex</code> and
     * <code>rowIndex</code>.
     * 
     * @param row
     * @param col
     * @param rowSpan
     * @param colSpan
     * @throws SpanException
     */
    public void combine( int row, int col, int rowSpan, int colSpan)throws SpanException;
    
    /**
     * Return true if can combine.
     * 
     * @param row
     * @param col
     * @param rowSpan
     * @param colSpan
     * @return true if can combine
     */
    public boolean  canCombine( int row, int col, int rowSpan, int colSpan);
    
    /**
     * Clear the span  at <code>columnIndex</code> and
     * <code>rowIndex</code>.
     *  
     * @param row
     * @param col
     * @throws SpanException
     */
    public void split(int row, int col)throws SpanException;
    
    /**
     * Return true if can split;
     * 
     * @param row
     * @param col
     * @return true if can split
     */
    public boolean canSplit(int row, int col);
    public void clear();
    /**
     * Returns true if the cell at <code>columnIndex</code> and
     * <code>rowIndex</code> is visible.
     * @param row
     * @param col
     * @return true if is visible
     * @throws SpanException
     */
    public boolean isVisible(int row, int col)throws SpanException;

    /**
     * Adds a listener to the list that is notified each time a change
     * to the span model occurs.
     *
     * @param l	the SpanModelListener
     */
    public void addSpanModelListener(SpanModelListener l);

    /**
     * Removes a listener from the list that is notified each time a
     * change to the span model occurs.
     *
     * @param l the SpanModelListener
     */
    public void removeSpanModelListener(SpanModelListener l);

}
