/**
 * 
 */
package misc.swing.table;


import WorkSpace.Table.TableSelectionModel;
import java.awt.Color;
import java.awt.Point;
import java.awt.Rectangle;
import java.util.Vector;

import javax.swing.JTable;
import javax.swing.ListSelectionModel;
import javax.swing.SwingConstants;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.TableColumnModelEvent;
import javax.swing.event.TableModelEvent;
import javax.swing.table.JTableHeader;
import javax.swing.table.TableColumn;
import javax.swing.table.TableColumnModel;
import javax.swing.table.TableModel;
import misc.swing.table.settings.Setting;
import misc.swing.table.settings.Styles;


/**
 * @author f00992
 *
 */
public class SpanTable extends JTable {
        protected TableSelectionModel tableSelectionModel;/////
	private static final long serialVersionUID = 2884357888554768774L;
	/**
	 * @see #getUIClassID
	 * @see #readObject
	 */
	//private static final String suiClassID = "es.aeat.eett.misc.swing.table.SpanTableUI";	
	protected Color horizontalLinesColor;
	protected Color verticalLinesColor;
	protected Setting setting;
	protected int horizontalTextAlignment=SwingConstants.LEADING;
	protected Styles styles;
	/*
	 * Almacenas una lista de estilos cuyos nombres
	 * siguen el convenio  row-col indicando que la celda 
	 * situada en la fila: row y la columna col usara ese estilo
	 */
	private Styles cellStyles=null;

	

	protected SpanModel spanModel;
	/**
	 * Constructor. 
	 */
	public SpanTable() {
		super();
		endInstallation();
	}

	/**
	 * @param dm
	 */
	public SpanTable(TableModel dm) {
		super(dm);
		endInstallation();
	}

	/**
	 * @param dm
	 * @param cm
	 */
	public SpanTable(TableModel dm, TableColumnModel cm) {
		super(dm, cm);
		endInstallation();
	}

	/**
	 * @param dm
	 * @param cm
	 * @param sm
	 */
	public SpanTable(TableModel dm, TableColumnModel cm, ListSelectionModel sm) {
		super(dm, cm, sm);
		endInstallation();
	}

	/**
	 * @param numRows
	 * @param numColumns
	 */
	public SpanTable(int numRows, int numColumns) {
		super(numRows, numColumns);
		endInstallation();
	}

	/**
	 * @param rowData
	 * @param columnNames
	 */
	public SpanTable(Vector<?> rowData, Vector<?> columnNames) {
		super(rowData, columnNames);
		endInstallation();
	}

	/**
	 * @param rowData
	 * @param columnNames
	 */
	public SpanTable(Object[][] rowData, Object[] columnNames) {
		super(rowData, columnNames);
		endInstallation();
	}	
	public SpanModel getDefaultSpanModel(){
		return new DefaultSpanModel(getModel());
	}
	
	protected void installSpanModel(){
		spanModel=getDefaultSpanModel();
	}
	protected void endInstallation(){
            horizontalLinesColor=getGridColor();
            verticalLinesColor=getGridColor();
            installSpanModel();
	    setUI(new SpanTableUI());
	    setUI(new SpanTableUI());
	    getTableHeader().setReorderingAllowed(false);
	    setCellSelectionEnabled(true);
            createDefaultTableSelectionModel();
	}
	
	/**
	 * @return Returns the setting.
	 */
	public Setting getSetting() {
		if(setting==null){
			setting= new Setting(false);
		}
		setting.setShowHorizontalLines(showHorizontalLines);
		setting.setShowVerticalLines(showVerticalLines);
		setting.setRowMargin(rowMargin);
		setting.setColumnMargin(getColumnModel().getColumnMargin());
		setting.setHorizontalLinesColor(horizontalLinesColor);
		setting.setVerticalLinesColor(verticalLinesColor);
		setting.setBackground(getBackground());
		setting.setForeground(getForeground());
		setting.setFont(getFont());
		setting.setHorizontalTextAlignment(horizontalTextAlignment);
		return setting;
	}

	/**
	 * @param setting The setting to set.
	 */
	public void setSetting(Setting setting) {
		if(setting!=null){
			this.setting = setting;
			showHorizontalLines=setting.isShowHorizontalLines();
			showVerticalLines=setting.isShowVerticalLines();
			rowMargin=setting.getRowMargin();
			getColumnModel().setColumnMargin(setting.getColumnMargin());
			horizontalLinesColor=setting.getHorizontalLinesColor();
			verticalLinesColor=setting.getVerticalLinesColor();
			//setBackground no tiene efecto si la tabla no es opaca
			setBackground(setting.getBackground());
			setForeground(setting.getForeground());
			setFont(setting.getFont());
			horizontalTextAlignment=setting.getHorizontalTextAlignment();
			styles=setting.getStyles();
		}
	}

	/**
	 * @return Returns the spanModel.
	 */
	public SpanModel getSpanModel() {
		return spanModel;
	}
//    public String getUIClassID() {
//        return suiClassID;
//    }
	
	protected JTableHeader createDefaultTableHeader() {
		//return super.createDefaultTableHeader();
		 //new JTableHeader(columnModel);
		return new SpanTableHeader(columnModel);
	}

	public Rectangle getCellRect(int row, int column, boolean includeSpacing) {
		if(spanModel!=null){
			try {
				CellPosition cp = spanModel.getVisibleCell(row, column);
				if(!cp.isEmpty()){
					Span span = spanModel.getSpanAt(cp.row, cp.col);
					if(span.spanR>1 || span.spanC>1){
						Rectangle r=super.getCellRect(cp.row, cp.col,true);
						for (int i = 1; i < span.spanR; i++) {
							r.height +=getRowHeight(cp.row+i);
						}
						int colMargin = getColumnModel().getColumnMargin();
						if(span.spanC>1){
							TableColumnModel tcm=getColumnModel();
							int nCol=tcm.getColumnCount();
							for (int i=1;i< span.spanC && (nCol>(cp.col+i)); i++) {
								TableColumn c = tcm.getColumn(cp.col+i);
								r.width += c.getWidth();
							}
						}
						if(!includeSpacing){
							int rm = getRowMargin();
							r.setBounds(r.x + colMargin/2
									, r.y + colMargin/2
									, r.width - colMargin
									, r.height - rm);
							
						}
						return r;
					}				
				}
			} catch (SpanException e1) {
			}
		}
		return super.getCellRect(row,column,includeSpacing);
	}
	
	private CellPosition getCellPositionAtPoint(Point point) {
		int row=super.rowAtPoint(point);
		int col=super.columnAtPoint(point);
		if ((row <0)||(getRowCount() <= row)) return CellPosition.EMPTY_CELL_POSITION;
		try {
			return spanModel.getVisibleCell(row, col);
		} catch (SpanException e) {
		}
		return CellPosition.EMPTY_CELL_POSITION;
	}
    @Override
	public int rowAtPoint(Point point) {
		CellPosition cellPosition=getCellPositionAtPoint(point);
		if(!cellPosition.isEmpty()){
			return cellPosition.getRow();
		}
		return -1;
	}
    @Override
	public int columnAtPoint(Point point) {
		CellPosition cellPosition=getCellPositionAtPoint(point);
		if(!cellPosition.isEmpty()){
			return cellPosition.getCol();
		}
		return -1;
	}
	
	public void columnSelectionChanged(ListSelectionEvent e) {
		valueChanged(e);
	}
	
	public void valueChanged(ListSelectionEvent e) {
		if (getRowCount() <= 0 || getColumnCount() <= 0) {
		    return;
		}
		int firstIndex = Math.min(0, Math.max(e.getFirstIndex()
				, getColumnCount()-1));
		int lastIndex = Math.min(getColumnCount()-1
				, Math.max(e.getLastIndex(), 0));
		int numCoumns = getColumnCount();
		int minIndex=firstIndex;
		int maxIndex=lastIndex;
		if(spanModel!=null && lastIndex < getRowCount()){
			for (int i=0;i<numCoumns;i++) {
				try {
					Span span = spanModel.getSpanAt(firstIndex, i);
					if(span.spanR<1){
						if((minIndex > minIndex+(span.spanR-1))){
							minIndex +=(span.spanR-1);
						}
						if(minIndex<=0){
							minIndex=0; break;
						}
						if(span.spanC-1>1){
							i += span.spanC-1;
						}
					}
				} catch (SpanException e1) {
				}
			}
			for (int i=0;i<numCoumns;i++) {
				try {
					Span span = spanModel.getSpanAt(lastIndex, i);
					if(span.spanR>1){
						if((maxIndex < maxIndex+(span.spanR-1))){
							maxIndex +=(span.spanR-1);
						}
						if(maxIndex >=numCoumns){
							maxIndex=numCoumns-1; break;
						}
						if(span.spanC>1){
							i += span.spanC-1;
						}
					}else if(span.spanR<0){
						CellPosition cp = spanModel.getVisibleCell(lastIndex, i);
						Span spanVisible = spanModel.getSpanAt(cp.row, cp.col);
						int offset=spanVisible.spanR-(lastIndex-cp.row);
						if(maxIndex < maxIndex+offset){
							maxIndex +=offset;
						}
						if(maxIndex >=numCoumns){
							maxIndex=numCoumns-1; break;
						}
						if(spanVisible.spanC>1){
							i += spanVisible.spanC-1;
						}
					}
				} catch (SpanException e1) {
				}
			}
		}
//	  System.out.println("Before->firstIndex:"+firstIndex+" lastIndex:"+lastIndex);
//	  System.out.println("\tLater->minIndex:"+minIndex+" maxIndex:"+maxIndex+" ...");
      Rectangle firstColumnRect = getCellRect(0, minIndex, false);
      Rectangle lastColumnRect = getCellRect(getRowCount()-1, maxIndex, false);
      Rectangle dirtyRegion = firstColumnRect.union(lastColumnRect);
      repaint(dirtyRegion);
      repaint();
	}

	public void tableChanged(TableModelEvent e) {
		// TODO actualizar spanModel 
		super.tableChanged(e);
		if(tableHeader instanceof SpanTableHeader){
			int h=getHeight();
			if(h>0){
				//int nRows=((SpanTableHeader)tableHeader).getDataModel().getRowCount();
				((SpanTableHeader)tableHeader)
					//.setRowHeight(h/nRows);
					.setRowHeight(SpanTableHeader.DEFAULT_ROW_HEIGHT);
			}
		}
	}
    public void columnAdded(TableColumnModelEvent e){
    	//TODO actualizar spanModel ????????
    	super.columnAdded(e);
    }
    public void columnRemoved(TableColumnModelEvent e){
    	//TODO actualizar spanModel ???????
    	super.columnRemoved(e);
    }
    public void columnMoved(TableColumnModelEvent e){
    	//TODO actualizar spanModel ???
    	super.columnMoved(e);
    }

	public void setModel(TableModel dataModel) {
		super.setModel(dataModel);
		if(spanModel!=null){
			spanModel.setTableModel(dataModel);
		}
//		if(spanModel!=null && !spanModel.getTableModel().equals(dataModel)){
//			spanModel.setTableModel(dataModel);
//		}
	}
    //
//	/**
//	 * @param spanModel The spanModel to set.
//	 */
//	public void setSpanModel(SpanModel spanModel) {
//		this.spanModel = spanModel;
//	}
//	public int convertColumnIndexToModel(int viewColumnIndex) {
//		return super.convertColumnIndexToModel(viewColumnIndex);
//	}
//	public int convertColumnIndexToView(int modelColumnIndex) {
//		return super.convertColumnIndexToView(modelColumnIndex);
//	}
	
	public void setGridColor(Color gridColor) {
		super.setGridColor(gridColor);
		horizontalLinesColor=gridColor;
		verticalLinesColor=gridColor;
	}

	/**
	 * @return Returns the horizontalLinesColor.
	 */
	public Color getHorizontalLinesColor() {
		return horizontalLinesColor;
	}

	/**
	 * @param horizontalLinesColor The horizontalLinesColor to set.
	 */
	public void setHorizontalLinesColor(Color horizontalLinesColor) {
		this.horizontalLinesColor = horizontalLinesColor;
	}

	/**
	 * @return Returns the verticalLinesColor.
	 */
	public Color getVerticalLinesColor() {
		return verticalLinesColor;
	}

	/**
	 * @param verticalLinesColor The verticalLinesColor to set.
	 */
	public void setVerticalLinesColor(Color verticalLinesColor) {
		this.verticalLinesColor = verticalLinesColor;
	}

	/**
	 * @return Returns the styles.
	 */
	public Styles getStyles() {
		return styles;
	}

	/**
	 * @param styles The styles to set.
	 */
	public void setStyles(Styles styles) {
		this.styles = styles;
	}

	/**
	 * @return Returns the cellStyles.
	 */
	public Styles getCellStyles() {
		return cellStyles;
	}

	/**
	 * @param cellStyles The cellStyles to set.
	 */
	public void setCellStyles(Styles cellStyles) {
		this.cellStyles = cellStyles;
	}

	/**
	 * @return Returns the horizontalTextAlignment.
	 */
	public int getHorizontalTextAlignment() {
		return horizontalTextAlignment;
	}

	/**
	 * @param horizontalTextAlignment The horizontalTextAlignment to set.
	 */
	public void setHorizontalTextAlignment(int horizontalTextAlignment) {
		this.horizontalTextAlignment = horizontalTextAlignment;
	}
	
	public void clearAllCombine(){
		//installSpanModel();
		if(spanModel!=null && spanModel.getTableModel()!=null){
			TableModel tm= spanModel.getTableModel();
			for (int i = 0; i < tm.getRowCount(); i++) {
				for (int j = 0; j < tm.getColumnCount(); j++) {
					if(spanModel.canSplit(i, j)){
						try {
							spanModel.split(i, j);
						} catch (SpanException e) {
						}
					}
				}
			}
		}else{
			installSpanModel();
		}
	}	
    @Override
        public boolean isCellEditable(int row,int column){
            return false;
        }
    @Override
   public boolean isCellSelected(int row, int column) {
    return tableSelectionModel.isSelected(row, convertColumnIndexToModel(column));
  }

  /**
    * Creates a default TableSelectionModel.
    */
  public void createDefaultTableSelectionModel() {
    TableSelectionModel tsm = new TableSelectionModel();
    setTableSelectionModel(tsm);
  }

  /**
    * same intention as setSelectionModel(ListSelectionModel newModel)
    */
  public void setTableSelectionModel(TableSelectionModel newModel) {
    //the TableSelectionModel shouldn't be null
    if (newModel == null) {
      throw new IllegalArgumentException("Cannot set a null TableSelectionModel");
    }

    //save the old Model
    TableSelectionModel oldModel = this.tableSelectionModel;
    //set the new Model
    this.tableSelectionModel = newModel;
    //The model needs to know how many columns are there
    newModel.setColumns(getColumnModel().getColumnCount());
    getModel().addTableModelListener(newModel);

    if (oldModel != null) {
      removePropertyChangeListener(oldModel);
    }
    addPropertyChangeListener(newModel);

    firePropertyChange("tableSelectionModel", oldModel, newModel);
  }

  /**
    * @return the current TableSelectionModel.
    */
  public TableSelectionModel getTableSelectionModel() {
    return tableSelectionModel;
  }

}
