package misc.swing.table;

import java.awt.Color;
import java.awt.Component;
import java.awt.Point;
import java.awt.Rectangle;
import java.awt.event.MouseEvent;

import javax.swing.JComponent;
import javax.swing.SizeSequence;
import javax.swing.SwingConstants;
import javax.swing.plaf.TableHeaderUI;
import javax.swing.table.DefaultTableModel;
import javax.swing.table.JTableHeader;
import javax.swing.table.TableCellRenderer;
import javax.swing.table.TableColumn;
import javax.swing.table.TableColumnModel;
import javax.swing.table.TableModel;
import misc.swing.table.settings.Setting;
import misc.swing.table.settings.Styles;



public class SpanTableHeader extends JTableHeader {
	private static final long serialVersionUID = 1L;
	
	protected SpanModel spanModel;

	protected boolean showHorizontalLines=false;

	protected boolean showVerticalLines=false;

	protected Color horizontalLinesColor;
	
	protected Color verticalLinesColor;

	protected int rowMargin=0;

	protected int colMargin=0;
	
	protected int horizontalTextAlignment=SwingConstants.CENTER;
	
	protected Styles styles;
	
	protected Setting setting;
	/*
	 * Almacenas una lista de estilos cuyos nombres
	 * siguen el convenio  row-col indicando que la celda 
	 * situada en la fila: row y la columna col usara ese estilo
	 */
	private Styles cellStyles=null;
	
    // The height in pixels of each row in the SpanTableHeader.
	public static final int DEFAULT_ROW_HEIGHT = 16;
    protected int  rowHeight =DEFAULT_ROW_HEIGHT;
    private SizeSequence rowModel;
	
	/**
	 * 
	 */
	public SpanTableHeader() {
		super();
		endInstallation();
	}

	/**
	 * @param cm
	 */
	public SpanTableHeader(TableColumnModel cm) {
		super(cm);
		endInstallation();
	}
	public void setHeaderRows(int nRows){
		spanModel=createDefaultSpanModel(nRows);
	}
	public void setReorderingAllowed(boolean b) {
		super.setReorderingAllowed(false);
	}
    public void setRowHeight(int rowHeight) {
        if (rowHeight <= 0) {
            throw new IllegalArgumentException("rowHeight less than 1");
        }
	    this.rowHeight = rowHeight;
		rowModel = null;
	    //resizeAndRepaint();
    }
    public void setRowHeight(int row, int rowHeight) {
        if (rowHeight <= 0) {
            throw new IllegalArgumentException("rowHeight less than 1");
        }
        getRowModel().setSize(row, rowHeight);
        //resizeAndRepaint();
    }
    public int getRowHeight(int row) {
    	return (rowModel == null) ? getRowHeight() : rowModel.getSize(row);
    }
    
    public int getRowHeight() {
        return rowHeight;
    }

    private SizeSequence getRowModel() {
	if (rowModel == null) {
	    rowModel = new SizeSequence(getRowCount(), getRowHeight());
	}
	return rowModel;
    }
	
	/**
	 * 
	 * @param tm
	 */
	public void setDataModel(TableModel tm){
		spanModel= new DefaultSpanModel(tm);
	}
	/**
	 * 
	 * @return el modelo de los datos que hay en la cabecera
	 */
	public TableModel getDataModel(){
		return spanModel.getTableModel();
	}
	
	/**
	 * @return Returns the colMargin.
	 */
	public int getColMargin() {
		return colMargin;
	}

	/**
	 * @param colMargin The colMargin to set.
	 */
	public void setColMargin(int colMargin) {
		this.colMargin = colMargin;
	}

	/**
	 * @return Returns the rowMargin.
	 */
	public int getRowMargin() {
		return rowMargin;
	}

	/**
	 * @param rowMargin The rowMargin to set.
	 */
	public void setRowMargin(int rowMargin) {
		this.rowMargin = rowMargin;
	}

	/**
	 * @return Returns the setting.
	 */
	public Setting getSetting() {
		if(setting==null){
			setting= new Setting(true);
		}
		setting.setShowHorizontalLines(showHorizontalLines);
		setting.setShowVerticalLines(showVerticalLines);
		setting.setRowMargin(rowMargin);
		setting.setColumnMargin(colMargin);
		setting.setHorizontalLinesColor(horizontalLinesColor);
		setting.setVerticalLinesColor(verticalLinesColor);
		setting.setBackground(getBackground());
		setting.setForeground(getForeground());
		setting.setFont(getFont());
		setting.setHorizontalTextAlignment(horizontalTextAlignment);
		return setting;
	}

	/**
	 * @param setting The setting to set.
	 */
	public void setSetting(Setting setting) {
		if(setting!=null){
			this.setting = setting;
			showHorizontalLines=setting.isShowHorizontalLines();
			showVerticalLines=setting.isShowVerticalLines();
			rowMargin=setting.getRowMargin();
			colMargin=setting.getColumnMargin();
			//getColumnModel().setColumnMargin(setting.getColumnMargin());
			horizontalLinesColor=setting.getHorizontalLinesColor();
			verticalLinesColor=setting.getVerticalLinesColor();
			//setBackground no tiene efecto si la tabla no es opaca
			setBackground(setting.getBackground());
			setForeground(setting.getForeground());
			setFont(setting.getFont());
			horizontalTextAlignment=setting.getHorizontalTextAlignment();
			styles=setting.getStyles();
		}
		
	}

	
	/**
	 * 
	 * @return the SpanModel
	 */
	public SpanModel getSpanModel(){
		return spanModel;
	}
	protected void endInstallation() {
		Setting ss=new Setting(true);
		horizontalLinesColor=ss.getHorizontalLinesColor();
		verticalLinesColor=ss.getVerticalLinesColor();
		spanModel=createDefaultSpanModel(1);
		setUI(new SpanTableHeaderUI());
		setReorderingAllowed(false);
	}
    public void setUI(TableHeaderUI ui){
    	if(ui instanceof SpanTableHeaderUI){
            super.setUI(ui);
    	}
    }
	
	protected SpanModel createDefaultSpanModel(int nRows){
		final TableColumnModel cm=getColumnModel();
		//ESTO ES UNA NIAPA PARA DEBUG.
		TableModel tm=new DefaultTableModel(nRows, cm.getColumnCount()){
			private static final long serialVersionUID = 1L;
			public Object getValueAt(int row, int col) { 
				if( row <getRowCount()-1){
					return "Hc:" + row + "-"+ col;
				}if(row == getRowCount()-1){
					TableColumn c = cm.getColumn(col);
					return c.getHeaderValue();
				}
				return null;
			}
		};
		SpanModel spanModel= new DefaultSpanModel(tm);
		
		return spanModel;
	}
	public Rectangle getCellRect(int row, int col) {
		Rectangle superR=getHeaderRect(col);
		Rectangle result=new Rectangle(superR);
		if(!superR.isEmpty()&& spanModel!=null){
			try {
				CellPosition cp = spanModel.getVisibleCell(row, col);
				if(!cp.isEmpty()){
					Span span = spanModel.getSpanAt(cp.row, cp.col);
					if(span.isVisible()){
						if(cp.col!=col){
							superR=getHeaderRect(cp.col);
							result=new Rectangle(superR);							
						}						
						if(!result.isEmpty()){
							int rowHeight=0;
							for(int i=0; i<span.spanR; i++){
								rowHeight += getRowHeight(cp.row+i);
							}
							result.height=rowHeight;
							for(int i=0; i<cp.row; i++){
								result.y += getRowHeight(i);
							}
							TableColumnModel tcm=getColumnModel();
							int nCols=tcm.getColumnCount();
							for (int i=1;i< span.spanC && cp.col+i<nCols; i++) {
								TableColumn c = tcm.getColumn(cp.col+i);
								result.width += c.getWidth();								
							}						
						}else{
							System.out.println("Super return empty Rectangle row:"+row+" col:"+cp.col);
							System.out.println(result);
						}
					}
				}
			} catch (SpanException e1) {
			}
		}
//		System.out.println("\nrow: "+row+" Col: "+col);
//		System.out.println(result);
		return result;
	}
	
	public String getToolTipText(MouseEvent event) {
		if(spanModel==null || spanModel.getTableModel().getRowCount()==1){
			return super.getToolTipText(event);
		}
		String toolTip = null;
		Point p = event.getPoint();
		CellPosition cp=getCellPositionAtPoint(p);
		if(cp!=CellPosition.EMPTY_CELL_POSITION){
			Object value=spanModel.getTableModel().getValueAt(cp.row,cp.col);
		    TableCellRenderer renderer =super.getDefaultRenderer();
		    Component component = renderer.getTableCellRendererComponent(
				      getTable(), value, false, false,
				      cp.row, cp.col);
		    if (component instanceof JComponent) {
				Rectangle cellRect = getCellRect(cp.row, cp.col);
				p.translate(-cellRect.x, -cellRect.y);
				MouseEvent newEvent = new MouseEvent(component, event.getID(),
							  event.getWhen(), event.getModifiers(),
							  p.x, p.y, event.getClickCount(),
							  event.isPopupTrigger());
	
				toolTip = ((JComponent)component).getToolTipText(newEvent);
		    }
		}
		if (toolTip == null)
			toolTip = getToolTipText();

		return toolTip;
	}
	private CellPosition getCellPositionAtPoint(Point point) {
		int col=super.columnAtPoint(point);
		if(col<0 ||getColumnModel().getColumnCount()<=col){
			return CellPosition.EMPTY_CELL_POSITION;
		}
		int row=rowAtPoint(point);
		if ((row <0)||( spanModel.getRowCount()<= row)) return CellPosition.EMPTY_CELL_POSITION;
		try {
			return spanModel.getVisibleCell(row, col);
		} catch (SpanException e) {
		}
		return CellPosition.EMPTY_CELL_POSITION;
	}	
	public int getRowCount(){
		return (spanModel==null)? 1:spanModel.getRowCount();
	}
	public int getColumnCount() {
		if(spanModel==null){
			return getColumnModel().getColumnCount();
		}else{
			return spanModel.getColumnCount();
		}
	}
	public int rowAtPoint(Point point) {
		int nRows=spanModel.getRowCount();
		//int rowHeight=(nRows!=0)? getHeight()/nRows:0;
		int height=0;
		for(int i=0; i<nRows; i++){
			//height +=rowHeight;
			height +=getRowHeight(i);
			if(height>point.y){
				return i;
			}
		}
		return -1;
	}
	public int visibleRowAtPoint(Point point) {
		int col=super.columnAtPoint(point);
		if(col>-1 && col < getColumnModel().getColumnCount()){
			int nRows=spanModel.getRowCount();
			//int rowHeight=(nRows!=0)? getHeight()/nRows:0;
			int height=0;
			for(int i=0; i<nRows; i++){
				//height +=rowHeight;
				height +=getRowHeight(i);
				if(height>point.y){
					CellPosition cellPosition;
					try {
						cellPosition = spanModel.getVisibleCell(i,col);
						if(!cellPosition.isEmpty()){
							return cellPosition.getRow();
						}
					} catch (SpanException e) {
						e.printStackTrace();
					}
				}
			}
		}
		return -1;
	}	
	
	public int columnAtPoint(Point point) {
		return super.columnAtPoint(point);
	}
	public int visiblecolumnAtPoint(Point point) {
		CellPosition cellPosition=getCellPositionAtPoint(point);
		if(!cellPosition.isEmpty()){
			return cellPosition.getCol();
		}
		return -1;
	}
	public boolean isShowHorizontalLines() {
		return showHorizontalLines;
	}

	public boolean isShowVerticalLines() {
		return showVerticalLines;
	}

	/**
	 * @param showHorizontalLines The showHorizontalLines to set.
	 */
	public void setShowHorizontalLines(boolean showHorizontalLines) {
		this.showHorizontalLines = showHorizontalLines;
	}

	/**
	 * @param showVerticalLines The showVerticalLines to set.
	 */
	public void setShowVerticalLines(boolean showVerticalLines) {
		this.showVerticalLines = showVerticalLines;
	}

	/**
	 * @return Returns the horizontalLinesColor.
	 */
	public Color getHorizontalLinesColor() {
		return horizontalLinesColor;
	}

	/**
	 * @param horizontalLinesColor The horizontalLinesColor to set.
	 */
	public void setHorizontalLinesColor(Color horizontalLinesColor) {
		this.horizontalLinesColor = horizontalLinesColor;
	}

	/**
	 * @return Returns the verticalLinesColor.
	 */
	public Color getVerticalLinesColor() {
		return verticalLinesColor;
	}

	/**
	 * @param verticalLinesColor The verticalLinesColor to set.
	 */
	public void setVerticalLinesColor(Color verticalLinesColor) {
		this.verticalLinesColor = verticalLinesColor;
	}

	/**
	 * @return Returns the styles.
	 */
	public Styles getStyles() {
		return styles;
	}

	/**
	 * @param styles The styles to set.
	 */
	public void setStyles(Styles styles) {
		this.styles = styles;
	}

	/**
	 * @return Returns the cellStyles.
	 */
	public Styles getCellStyles() {
		return cellStyles;
	}

	/**
	 * @param cellStyles The cellStyles to set.
	 */
	public void setCellStyles(Styles cellStyles) {
		this.cellStyles = cellStyles;
	}

	/**
	 * @return Returns the horizontalTextAlignment.
	 */
	public int getHorizontalTextAlignment() {
		return horizontalTextAlignment;
	}

	/**
	 * @param horizontalTextAlignment The horizontalTextAlignment to set.
	 */
	public void setHorizontalTextAlignment(int horizontalTextAlignment) {
		this.horizontalTextAlignment = horizontalTextAlignment;
	}
	public void clearAllCombine(){
		if(spanModel!=null){
			spanModel= new DefaultSpanModel(spanModel.getTableModel());
		}
	}	
	
}
