package misc.swing.table;



import java.awt.Color;
import java.awt.Component;
import java.awt.Cursor;
import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.Point;
import java.awt.Rectangle;
import java.awt.event.MouseEvent;

import javax.swing.JComponent;
import javax.swing.event.MouseInputListener;
import javax.swing.plaf.basic.BasicTableHeaderUI;
import javax.swing.table.TableCellRenderer;
import javax.swing.table.TableColumn;
import javax.swing.table.TableColumnModel;



public class SpanTableHeaderUI extends BasicTableHeaderUI {
	protected SpanTableHeader spanHeader;

    /**
     * Oberwrite the mouse listener for the JTable.
     */
	private static Cursor resizeCursor2 = Cursor.getPredefinedCursor(Cursor.E_RESIZE_CURSOR);
    protected MouseInputListener createMouseInputListener() {
        return new MouseInputHandler(){
            public void mouseMoved(MouseEvent e) {
                if (canResize2(getResizingColumn2(e.getPoint()))!=
            		(header.getCursor() == resizeCursor2)) {
                	super.mouseMoved(e);
                }
           }
        	private boolean canResize2(TableColumn column) {
        	    return (column != null) && header.getResizingAllowed() && column.getResizable();
        	}
        	private TableColumn getResizingColumn2(Point p) {
        		int col=header.columnAtPoint(p);
        		int row=((SpanTableHeader) header).rowAtPoint(p);
        		//if (col == -1 || ((SpanTableHeader) header).getColumnCount()==col-1) {
        		if (col == -1 ){
        			return null; 
        		}
        		Rectangle r;
        		if(row==((SpanTableHeader) header).getRowCount()-1){
            		r = header.getHeaderRect(col);
        		}else{
        			r=((SpanTableHeader) header).getCellRect(row, col);
        		}
        		r.grow(-3, 0); 
        		if (r.contains(p)) { 
        			return null; 
        		}
        		int midPoint = r.x + r.width/2; 
        		int columnIndex;
        		if( header.getComponentOrientation().isLeftToRight() ) {
        			columnIndex = (p.x < midPoint) ? col - 1 : col; 
        		} else {
        			columnIndex = (p.x < midPoint) ? col : col - 1; 
        		}
        		if (columnIndex == -1) { 
        			return null; 
        		}
        		return header.getColumnModel().getColumn(columnIndex); 
        	}
        };
    }
    
	public void paint(Graphics g, JComponent c) {
		if( header instanceof SpanTableHeader){
		if (header.getColumnModel() == null) return;		
		spanHeader=(SpanTableHeader) header;

		boolean ltr = header.getComponentOrientation().isLeftToRight();
		
		Rectangle clip = g.getClipBounds(); 
		Point left = clip.getLocation();
		Point right = new Point( clip.x + clip.width - 1, clip.y );
		TableColumnModel cm = header.getColumnModel(); 
		int cMin = header.columnAtPoint( ltr ? left : right );
		int cMax = header.columnAtPoint( ltr ? right : left );
		int rMin=0;
		int rMax = spanHeader.getRowCount()-1;
		if (cMin == -1) {
			cMin =  0;
		}
		if (cMax == -1) {
			cMax = cm.getColumnCount()-1;  
		}
		//System.out.println("--cMin="+cMin+" cMax="+cMax); //Debug
		paintCells(g, rMin, rMax, cMin, cMax);
		}else{
			super.paint(g,c);
		}
	}
	private void paintCells(Graphics g, int rMin, int rMax, int cMin, int cMax) {
		if(header.getComponentOrientation().isLeftToRight()){
			for(int row = rMin; row <= rMax; row++) {
				paintRowLeftToRight(g,row,cMin, cMax);
			}
		}else{
			for(int row = rMin; row <= rMax; row++) {
				paintRowRightToLeft(g,row,cMin, cMax);
			}
		}
		rendererPane.removeAll();
	}

	private void paintRowRightToLeft(Graphics g, int row, int cMin, int cMax) {
		//TODO sin implementar trabaja pero mal.
		paintRowLeftToRight(g, row, cMin, cMax);
		
	}
	private void paintRowLeftToRight(Graphics g, int row, int cMin, int cMax) {
		Rectangle rectangle = g.getClipBounds();
		boolean drawnRow  = false;
		
		SpanModel spanModel=spanHeader.getSpanModel();
		
		//System.out.print("\nRow: "+row);  // debug
		for (int col = cMin; col <= cMax; col++) {
			Rectangle cellRect = spanHeader.getCellRect(row,col);
			int iRow = row;
			int iCol = col;
			try {
				if (spanModel.isVisible(row,col)) {
					iRow = row;
					iCol = col;
					//System.out.print("\t"+col);  // debug
				}else{
					Span span=spanModel.getSpanAt(row,col);
					iRow = row + span.spanR;
					iCol = col + span.spanC;
					//System.out.print(span);  // debug
				}
			} catch (SpanException e) {
				//e.printStackTrace();
				//System.out.println("SpanTable UI Erro:"+e.getMessage());//Debbug
			}
			if (cellRect.intersects(rectangle)) {
				drawnRow = true;
				paintCell(g, cellRect, iRow, iCol);
			} else {
				if (drawnRow) break;
			} 
		}
	}
	
	private void paintCell(Graphics g, Rectangle cellRect, int row, int col) {
		// 1 pixel for the grill
		//int spacingHeight =1;//header.getRowMargin()+1;
		int spacingHeight =spanHeader.getRowMargin()+1;
		//int spacingWidth  = header.getColumnModel().getColumnMargin()+1;
		int spacingWidth  = spanHeader.getColMargin()+1;
		
		cellRect.width ++;
		cellRect.height ++;
		if(spacingHeight>1 || spacingWidth>1){
			cellRect.x --;
			cellRect.y--;
			paintCellBounds(g, cellRect);
//			cellRect.x ++;
//			cellRect.y++;
		}
		
		cellRect.setBounds(
				cellRect.x + spacingWidth/2
				,cellRect.y+ spacingHeight/2
				,cellRect.width - spacingWidth
				,cellRect.height - spacingHeight);
		
		TableColumn aColumn = header.getColumnModel().getColumn(col); 
		TableCellRenderer renderer = aColumn.getHeaderRenderer(); 
		if (renderer == null) { 
			renderer = header.getDefaultRenderer(); 
		}
		Object oValue=spanHeader.getDataModel().getValueAt(row, col);
		Component component= renderer.getTableCellRendererComponent(header.getTable(), 
				oValue, false, false, 
				row, col);
		
		rendererPane.paintComponent(g, component, header, cellRect.x, cellRect.y,
                            cellRect.width, cellRect.height, true);
	}    
	private void paintCellBounds(Graphics g, Rectangle r){
		if(spanHeader.isShowHorizontalLines()){
			Color c = g.getColor();
			g.setColor(spanHeader.getHorizontalLinesColor());
			if (r.height == 0 || r.width == 0) {
				g.drawLine(r.x, r.y, r.x + r.width, r.y + r.height);
			} else {
				//top line //
				g.drawLine(r.x, r.y, r.x + r.width+ - 1, r.y);
				//bottom line //
				//g.drawLine(r.x + r.width, r.y + r.height, r.x + 1, r.y + r.height);
			}
			g.setColor(c);
		}
		if(spanHeader.isShowVerticalLines()){
			Color c = g.getColor();
			g.setColor(spanHeader.getVerticalLinesColor());
			if (r.height == 0 || r.width == 0) {
				g.drawLine(r.x, r.y, r.x + r.width, r.y + r.height);
			} else {
				//right line //
				//g.drawLine(r.x + r.width, r.y, r.x + r.width, r.y + r.height - 1);
				// left line//
				g.drawLine(r.x, r.y + r.height, r.x, r.y + 1);
			}
			g.setColor(c);
		}
	}
	
	
	public Dimension getPreferredSize(JComponent c) {
		Dimension superD=super.getPreferredSize(c);
		Dimension d=new Dimension(superD);
		if(header instanceof SpanTableHeader){
			SpanTableHeader spanHeader=(SpanTableHeader) header;
			SpanModel spanModel=spanHeader.getSpanModel();
			if(spanModel!=null){
				int nRows=spanModel.getRowCount();
				//d.height=d.height*nRows;
				d.height=0;
				for(int i=0; i<nRows; i++){
					d.height +=spanHeader.getRowHeight(i);
				}				
			}
			if (d.width > Integer.MAX_VALUE) {
				d.width = Integer.MAX_VALUE;
			}			
		}
		return d;
	}
}
