package misc.swing.table;





import WorkSpace.Table.TableSelectionModel;
import java.awt.Color;
import java.awt.Component;
import java.awt.Graphics;
import java.awt.Point;
import java.awt.Rectangle;

import javax.swing.JComponent;
import javax.swing.plaf.basic.BasicTableUI;
import javax.swing.table.TableCellRenderer;
import java.awt.Point;
import java.awt.event.MouseEvent;

import javax.swing.JComponent;
import javax.swing.JTable;
import javax.swing.SwingUtilities;
import javax.swing.table.TableCellEditor;
import javax.swing.plaf.basic.BasicTableUI;
import javax.swing.plaf.ComponentUI;
import javax.swing.event.MouseInputListener;


public class SpanTableUI extends BasicTableUI {
	
	public void paint(Graphics g, JComponent c) {
		if (table.getRowCount() <= 0 || table.getColumnCount() <= 0) {
			return;
		}
		Rectangle clip = g.getClipBounds();
		Point upperLeft = clip.getLocation();
		Point lowerRight = new Point(clip.x + clip.width - 1, clip.y + clip.height - 1);
		int rMin = table.rowAtPoint(upperLeft);
		int rMax = table.rowAtPoint(lowerRight);
		
		if (rMin == -1) {
			rMin = 0;
		}
		if (rMax == -1 ) {
			rMax = table.getRowCount()-1;
		}
		
		boolean ltr = table.getComponentOrientation().isLeftToRight();
		int cMin = table.columnAtPoint(ltr ? upperLeft : lowerRight); 
		int cMax = table.columnAtPoint(ltr ? lowerRight : upperLeft);        
		if (cMin == -1) {
			cMin = 0;
		}
		if (cMax == -1 ) {
			cMax = table.getColumnCount()-1;
		}
		//System.out.println("cMax="+cMax+" rMax="+rMax);
		SpanModel spanModel=((SpanTable)table).getSpanModel();
		if(spanModel!=null){
			try {
				//System.out.println();
				CellPosition cp = spanModel.getVisibleCell(rMax, cMax);
				Span span = spanModel.getSpanAt(cp.row, cp.col);
				if(span.spanC>1){
					int i=cMax;
					i += span.spanC-1;
					if(i>table.getColumnCount()-1){
						i=table.getColumnCount()-1;
					}
					if(cMax<i){
						//System.out.println("cMax old="+cMax+" cMax="+i);
						cMax=i;
					}
				}
				cp = spanModel.getVisibleCell(rMax, cMax);
				 span = spanModel.getSpanAt(cp.row, cp.col);
				if(span.spanR>1){
					int i=rMax;
					i +=span.spanR-1;
					if(i>table.getRowCount()-1){
						i=table.getRowCount()-1;
					}
					if(rMax<i){
						//System.out.println("rMax old="+rMax+" rMax="+i);
						rMax=i;
					}
				}
			} catch (SpanException e) {
				e.printStackTrace();
			}
		}
		paintCells(g, rMin, rMax, cMin, cMax);
	}
	private void paintCells(Graphics g, int rMin, int rMax, int cMin, int cMax) {
		if(table.getComponentOrientation().isLeftToRight()){
			for(int row = rMin; row <= rMax; row++) {
				paintRowLeftToRight(g,row,cMin, cMax);
			}
		}else{
			for(int row = rMin; row <= rMax; row++) {
				paintRowRightToLeft(g,row,cMin, cMax);
			}
		}
		rendererPane.removeAll();
	}

	private void paintRowRightToLeft(Graphics g, int row, int cMin, int cMax) {
		//TODO sin implementar trabaja pero mal.
		paintRowLeftToRight(g, row, cMin, cMax);
		
	}
	private void paintRowLeftToRight(Graphics g, int row, int cMin, int cMax) {
		Rectangle rectangle = g.getClipBounds();
		boolean drawnRow  = false;		
		SpanModel spanModel=((SpanTable)table).getSpanModel();
		
		//System.out.print("\nRow: "+row);  // debug
		for (int col = cMin; col <= cMax; col++) {
			Rectangle cellRect = table.getCellRect(row,col,true);
			int iRow = row;
			int iCol = col;
			try {
				if (spanModel.isVisible(row,col)) {
					iRow = row;
					iCol = col;
					//System.out.print("\t"+col);  // debug
				}else{
					Span span=spanModel.getSpanAt(row,col);
					iRow = row + span.spanR;
					iCol = col + span.spanC;
					//System.out.print(span);  // debug
				}
			} catch (SpanException e) {
				//e.printStackTrace();
				System.out.println("SpanTable UI Erro:"+e.getMessage());//Debbug
			}
			if (cellRect.intersects(rectangle)) {
				drawnRow = true;
				paintCell(g, cellRect, iRow, iCol);
			} else {
				if (drawnRow) break;
			} 
		}
	}
	
	private void paintCell(Graphics g, Rectangle cellRect, int row, int column) {
		// 1 pixel for the grill
		int spacingHeight = table.getRowMargin()+1;
		int spacingWidth  = table.getColumnModel().getColumnMargin()+1;
		
		//Color c = g.getColor();
		cellRect.x -=1;
		cellRect.width +=1;
		//r.y -=1;
		cellRect.height +=1;
		paintCellBounds(g, cellRect);
		//g.setColor(c);
		
		cellRect.setBounds(
				cellRect.x + spacingWidth/2
				, cellRect.y+ spacingHeight/2
				,cellRect.width - spacingWidth
				,cellRect.height - spacingHeight);
		
		if (table.isEditing() && table.getEditingRow()==row &&
			table.getEditingColumn()==column) {
			Component component = table.getEditorComponent();
			component.setBounds(cellRect);
			component.validate();
		}else {
			TableCellRenderer renderer = table.getCellRenderer(row, column);
			Component component = table.prepareRenderer(renderer, row, column);
			
			if (component.getParent() == null) {
				rendererPane.add(component);
			}
			rendererPane.paintComponent(g, component, table, cellRect.x, cellRect.y,
					cellRect.width, cellRect.height, true);
		}
	}    
	private void paintCellBounds(Graphics g, Rectangle r){
		if(table.getShowHorizontalLines()){
			Color c = g.getColor();
			if(table instanceof SpanTable){
				g.setColor(((SpanTable)table).getHorizontalLinesColor());
			}else{
				g.setColor(table.getGridColor());
			}
			if (r.height == 0 || r.width == 0) {
				g.drawLine(r.x, r.y, r.x + r.width, r.y + r.height);
			} else {
				//top line //
				g.drawLine(r.x, r.y, r.x + r.width+ - 1, r.y);
				//bottom line //
				//g.drawLine(r.x + r.width, r.y + r.height, r.x + 1, r.y + r.height);
			}
			g.setColor(c);
		}
		if(table.getShowVerticalLines()){
			Color c = g.getColor();
			if(table instanceof SpanTable){
				g.setColor(((SpanTable)table).getVerticalLinesColor());
			}else{
				g.setColor(table.getGridColor());
			}
			if (r.height == 0 || r.width == 0) {
				g.drawLine(r.x, r.y, r.x + r.width, r.y + r.height);
			} else {
				//right line //
				//g.drawLine(r.x + r.width, r.y, r.x + r.width, r.y + r.height - 1);
				// left line//
				g.drawLine(r.x, r.y + r.height, r.x, r.y + 1);
			}
			g.setColor(c);
		}
	}
public static ComponentUI createUI(JComponent c) {
    return new AnySelectionTableUI();
  }

  protected MouseInputListener createMouseInputListener() {
    return new AnySelectionMouseInputHandler();
  }

  /**
    * to get access to the table from the inner class MyMouseInputHandler
    */
  protected JTable getTable() {
    return table;
  }

  /**
    * updates the TableSelectionModel.
    */
  protected void updateTableSelectionModel(int row, int column,
                                   boolean ctrlDown, boolean shiftDown) {

    SpanTable t = (SpanTable)getTable();
    column = t.convertColumnIndexToModel(column);
    TableSelectionModel tsm = t.getTableSelectionModel();

    int anchorIndex = tsm.getListSelectionModelAt(column).getAnchorSelectionIndex();

    if (ctrlDown) {
      if (tsm.isSelected(row, column)) {
        tsm.removeSelection(row, column);
      } else {
        tsm.addSelection(row, column);
      }
    } else if ((shiftDown) && (anchorIndex != -1)){
      tsm.setSelectionInterval(anchorIndex, row, column);
    } else {
      tsm.setSelection(row, column);
    }
  } //updateTableSelectionModel()

  /**
    * Almost the same implementation as its super class.
    * Except updating the TableSelectionModel rather than the
    * default ListSelectionModel.
    */
  //Some methods which are called in the super class are private.
  //Thus I couldn't call them. Calling the method of the super
  //class itself should do it, but you never know. Sideeffects may occur...
  public class AnySelectionMouseInputHandler extends MouseInputHandler {

    public void mousePressed(MouseEvent e) {
      super.mousePressed(e);

	    if (SwingUtilities.isLeftMouseButton(e)) {
                 return;
	    }
	    if (SwingUtilities.isMiddleMouseButton(e)) {
                 return;
	    }

      /*if (phantomMousePressed == true) {
        return;
      }
      phantomMousePressed = true;*/

      Point p = e.getPoint();
      int row = getTable().rowAtPoint(p);
      int column = getTable().columnAtPoint(p);
	    // The autoscroller can generate drag events outside the Table's range.
      if ((column == -1) || (row == -1)) {
        return;
      }

       /* Adjust the selection if the event was not forwarded 
	     * to the editor above *or* the editor declares that it 
	     * should change selection even when events are forwarded 
	     * to it.
	     */
	    // PENDING(philip): Ought to convert mouse event, e, here. 
	    //if (!repostEvent || table.getCellEditor().shouldSelectCell(e)) {
      TableCellEditor tce = getTable().getCellEditor();
      if ((tce==null) || (tce.shouldSelectCell(e))) {
        getTable().requestFocus();
        //setValueIsAdjusting(true);
        //if (getTable().getValueAt(row, column) != null) {
          updateTableSelectionModel(row, column, e.isControlDown(), e.isShiftDown());
          //need to do this because JTable is not painted properly when
          //user does Shift-Click on a column where a not selected
          //cell is the anchor ...
        //  if (e.isShiftDown())
            getTable().repaint();
        //}
      }
    }//mousePressed()
  }

}
