package misc.swing.table.settings;



import java.awt.Color;
import java.awt.Font;
import javax.swing.SwingConstants;
import javax.swing.UIManager;
/**
 * Almacena la confinguracion de spanTable;
 * @author f00992
 *
 */
public class Setting {
	private boolean showHorizontalLines=true;
	private boolean showVerticalLines=true;
	private int rowMargin=1;
	private int columnMargin=1;
	private Color horizontalLinesColor;
	private Color verticalLinesColor;
	private Color background;
	private Color foreground;
	private Font font;
	private Styles styles;
	private int horizontalTextAlignment;
	//private String path;
	
	private boolean isHeader;	
	private boolean isRowHeader;
	
	public Setting() {
		this(false,false);
	}
	public Setting(boolean isHeader) {
		this(isHeader,false);
	}
	public Setting(boolean isHeader, boolean isRowHeader) {
		this.isHeader=isHeader;
		this.isRowHeader=isRowHeader;
		loadDefault(isHeader, isRowHeader);
	}
	public void loadDefault(boolean isHeader, boolean isRowHeader) {
		if(isHeader){
			showHorizontalLines=false;
			showVerticalLines=false;
			rowMargin=0;
			columnMargin=0;
			background=UIManager.getColor("TableHeader.background");
			foreground=UIManager.getColor("TableHeader.foreground");
			font=UIManager.getFont("TableHeader.font");
			if(isRowHeader){
				horizontalTextAlignment=SwingConstants.LEADING;
			}else{
				horizontalTextAlignment=SwingConstants.CENTER;
			}
		}else{
			showHorizontalLines=true;
			showVerticalLines=true;
			rowMargin=1;
			columnMargin=1;
			if(isRowHeader){
				background=UIManager.getColor("TableHeader.background");
				foreground=UIManager.getColor("TableHeader.foreground");
				font=UIManager.getFont("TableHeader.font");
				horizontalTextAlignment=SwingConstants.LEADING;
			}else{
				background=UIManager.getColor("Table.background");
				foreground=UIManager.getColor("Table.foreground");
				font=UIManager.getFont("Table.font");
				horizontalTextAlignment=SwingConstants.TRAILING;
			}
		}
		horizontalLinesColor=UIManager.getColor("Table.gridColor");
		verticalLinesColor=UIManager.getColor("Table.gridColor");
	}
	public void loadDefault() {
		loadDefault(isHeader,isRowHeader);
	}

	/**
	 * @return Returns the background.
	 */
	public Color getBackground() {
		return background;
	}


	/**
	 * @param background The background to set.
	 */
	public void setBackground(Color background) {
		this.background = background;
	}


	/**
	 * @return Returns the columnMargin.
	 */
	public int getColumnMargin() {
		return columnMargin;
	}


	/**
	 * @param columnMargin The columnMargin to set.
	 */
	public void setColumnMargin(int columnMargin) {
		this.columnMargin = columnMargin;
	}


	/**
	 * @return Returns the font.
	 */
	public Font getFont() {
		return font;
	}


	/**
	 * @param font The font to set.
	 */
	public void setFont(Font font) {
		this.font = font;
	}


	/**
	 * @return Returns the foreground.
	 */
	public Color getForeground() {
		return foreground;
	}


	/**
	 * @param foreground The foreground to set.
	 */
	public void setForeground(Color foreground) {
		this.foreground = foreground;
	}


	/**
	 * @return Returns the horizontalLinesColor.
	 */
	public Color getHorizontalLinesColor() {
		return horizontalLinesColor;
	}


	/**
	 * @param horizontalLinesColor The horizontalLinesColor to set.
	 */
	public void setHorizontalLinesColor(Color horizontalLinesColor) {
		this.horizontalLinesColor = horizontalLinesColor;
	}


	/**
	 * @return Returns the rowMargin.
	 */
	public int getRowMargin() {
		return rowMargin;
	}


	/**
	 * @param rowMargin The rowMargin to set.
	 */
	public void setRowMargin(int rowMargin) {
		this.rowMargin = rowMargin;
	}


	/**
	 * @return Returns the showHorizontalLines.
	 */
	public boolean isShowHorizontalLines() {
		return showHorizontalLines;
	}


	/**
	 * @param showHorizontalLines The showHorizontalLines to set.
	 */
	public void setShowHorizontalLines(boolean showHorizontalLines) {
		this.showHorizontalLines = showHorizontalLines;
	}


	/**
	 * @return Returns the showVerticalLines.
	 */
	public boolean isShowVerticalLines() {
		return showVerticalLines;
	}


	/**
	 * @param showVerticalLines The showVerticalLines to set.
	 */
	public void setShowVerticalLines(boolean showVerticalLines) {
		this.showVerticalLines = showVerticalLines;
	}


	/**
	 * @return Returns the verticalLinesColor.
	 */
	public Color getVerticalLinesColor() {
		return verticalLinesColor;
	}


	/**
	 * @param verticalLinesColor The verticalLinesColor to set.
	 */
	public void setVerticalLinesColor(Color verticalLinesColor) {
		this.verticalLinesColor = verticalLinesColor;
	}


	/**
	 * @return Returns the horizontalTextAlignment.
	 */
	public int getHorizontalTextAlignment() {
		return horizontalTextAlignment;
	}

	public boolean isDefaultSetting(){
		Setting defaultSetting=new Setting(isHeader, isRowHeader);
		if(defaultSetting.showHorizontalLines!=showHorizontalLines
				||defaultSetting.showVerticalLines!=showVerticalLines
				||defaultSetting.rowMargin!=rowMargin
				||defaultSetting.columnMargin!=columnMargin
				||!defaultSetting.horizontalLinesColor.equals(horizontalLinesColor)
				||!defaultSetting.verticalLinesColor.equals(verticalLinesColor)
				||!defaultSetting.background.equals(background)
				||!defaultSetting.foreground.equals(foreground)
				||!defaultSetting.font.equals(font)
				||defaultSetting.horizontalTextAlignment!=horizontalTextAlignment){
			
			return false;
		}
		
		return true;
	}

	/**
	 * @param horizontalTextAlignment The horizontalTextAlignment to set.
	 */
	public void setHorizontalTextAlignment(int horizontalTextAlignment) {
        if ((horizontalTextAlignment == SwingConstants.LEFT) ||
                (horizontalTextAlignment == SwingConstants.CENTER) ||
                (horizontalTextAlignment == SwingConstants.RIGHT) ||
                (horizontalTextAlignment == SwingConstants.LEADING) ||
                (horizontalTextAlignment == SwingConstants.TRAILING)) {
    		this.horizontalTextAlignment = horizontalTextAlignment;
            }
            else {
                throw new IllegalArgumentException("invalid horizontalTextAlignment");
            }
	}

	/**
	 * @return Returns the isHeader.
	 */
	public boolean isHeader() {
		return isHeader;
	}
	
	/**
	 * @return Returns the isRowHeader.
	 */
	public boolean isRowHeader() {
		return isRowHeader;
	}
	/**
	 * @return Returns the styles.
	 */
	public Styles getStyles() {
		return styles;
	}
	/**
	 * @param styles The styles to set.
	 */
	public void setStyles(Styles styles) {
		this.styles = styles;
	}

	public boolean equals(Object obj) {
		if(obj==this){
			return true;
		}else if(obj instanceof Setting){
			Setting s=(Setting)obj;
			if(s.isHeader!=isHeader){
				return false;
			}
			if(s.isRowHeader!=isRowHeader){
				return false;
			}
			if(s.showHorizontalLines!=showHorizontalLines){
				return false;
			}			
			if(s.showVerticalLines!=showVerticalLines){
				return false;
			}
			if(s.rowMargin!=rowMargin){
				return false;
			}
			if(s.columnMargin!=columnMargin){
				return false;
			}
			if(s.horizontalTextAlignment!=horizontalTextAlignment){
				return false;
			}
			if(horizontalLinesColor!=null ){
				if(!horizontalLinesColor.equals(s.horizontalLinesColor)){
					return false;
				}
			}else if(s.horizontalLinesColor!=null){
				return false;
			}
			if(verticalLinesColor!=null ){
				if(!verticalLinesColor.equals(s.verticalLinesColor)){
					return false;
				}
			}else if(s.verticalLinesColor!=null){
				return false;
			}
			if(background!=null ){
				if(!background.equals(s.background)){
					return false;
				}
			}else if(s.background!=null){
				return false;
			}
			if(foreground!=null ){
				if(!foreground.equals(s.foreground)){
					return false;
				}
			}else if(s.foreground!=null){
				return false;
			}
			if(font!=null ){
				if(!font.equals(s.font)){
					return false;
				}
			}else if(s.font!=null){
				return false;
			}
			
			if(styles!=null && !(styles.equals(s.styles))){
				return false;
			}else if(styles==null && s.styles!=null){
				return false;
			}
			return true;
		}
		return false;
	}
	public Object clone(){
		Setting setting=new Setting(isHeader, isRowHeader);
		setting.showHorizontalLines=showHorizontalLines;
		setting.showVerticalLines=showVerticalLines;
		setting.rowMargin=rowMargin;
		setting.columnMargin=columnMargin;
		setting.horizontalLinesColor=new Color(horizontalLinesColor.getRGB());
		setting.verticalLinesColor=new Color(verticalLinesColor.getRGB());
		setting.background=new Color(background.getRGB());
		setting.foreground=new Color(foreground.getRGB());
		setting.font=new Font(font.getFontName(),font.getStyle(), font.getSize());
		if(styles!=null){
			setting.styles=(Styles) styles.clone();
		}
		setting.horizontalTextAlignment=horizontalTextAlignment;
		return setting;
	}
	
//	/**
//	 * @return Returns the path.
//	 */
//	public String getPath() {
//		return path;
//	}
//	/**
//	 * @param path The path to set.
//	 */
//	public void setPath(String path) {
//		this.path = path;
//	}
	
	

}
