package misc.swing.table.settings;



import java.awt.Color;
import java.awt.Font;
import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.File;

import javax.swing.AbstractAction;
import javax.swing.BorderFactory;
import javax.swing.DefaultComboBoxModel;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JColorChooser;
import javax.swing.JComboBox;
import javax.swing.JDialog;
import javax.swing.JFileChooser;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTextField;
import javax.swing.SwingUtilities;
import javax.swing.border.Border;
import misc.swing.combofont.ComboFontName;
import misc.swing.combofont.ComboFontSize;
import misc.swing.combofont.ComboFontStyle;
import misc.swing.combofont.ComboHorizontalAlignment;
import misc.swing.combofont.StyleSupport;
import misc.util.IntegerDocument;
import misc.util.Util;



public class SettingPanel extends JPanel {
	private static final long serialVersionUID = 1L;
	
	private JComboBox cSelector;	
	private String[] sSelector={"Table", "Header", "RowHeader"};	
	private JCheckBox  checkShowHorizontalLines;
	private JCheckBox  checkShowVerticalLines;
	private JTextField textRowMargin;
	private JTextField textColMargin;
	private JButton bHorizontalLinesColor;
	private JButton bVerticalLinesColor;
	private JButton bBackground;
	private JButton bForeground;
	private ComboFontName cFont;
	private ComboFontStyle cFontStyle;
	private ComboFontSize cFontSize;
	private ComboHorizontalAlignment cTextPosition;
	private JTextField textStyle;
	private JCheckBox  checkApplyStyles;
	private JButton bStyle;
	private JButton bDefault;
	private JButton bOK;
	private JButton bCancel;
	private TableSetting tableSetting;
	private Setting activeSetting;
	private boolean accept=false;
	
	/*
	private int horizontalTextPosition;
	private File stylesFile;
	 */
	public SettingPanel(TableSetting tableSetting) {
		super();		
		initPanel(tableSetting);
	}

	private void initPanel(TableSetting tableSetting) {
		this.tableSetting=(TableSetting) tableSetting.clone();
		//this.tableSetting=tableSetting;
		setLayout(new GridLayout(10,2));
		createComponets();
		loadComponets(tableSetting);
		add(cSelector);
		add(new JLabel());
		add(checkShowHorizontalLines);
		add(checkShowVerticalLines);
		add(textRowMargin);
		add(textColMargin);
		add(bHorizontalLinesColor);
		add(bVerticalLinesColor);
		add(bBackground);
		add(bForeground);
		add(cFont);
		add(cFontStyle);
		add(cFontSize);
		//add(new JLabel());
		add(cTextPosition);
		add(textStyle);
		add(bStyle);
		
		//add(new JLabel());
		add(checkApplyStyles);
		add(bDefault);
		
		add(bOK);
		add(bCancel);
		
	}

	private void createComponets() {
		cSelector = new JComboBox(new DefaultComboBoxModel(sSelector));
		cSelector.setBorder(createTitleBorder("Selector"));
		cSelector.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent e) {
				loadActiveSetting();
				switch (cSelector.getSelectedIndex()) {
				case 0:
					activeSetting=tableSetting.getDataSetting();
					break;
				case 1:
					activeSetting=tableSetting.getHeaderSetting();
					break;
				case 2:
					activeSetting=tableSetting.getRowHeaderSetting();
					break;
				}
				loadComponets();
			}			
		});
		activeSetting=tableSetting.getDataSetting();
		setBorder(createTitleBorder("Setting:"));
		checkShowHorizontalLines=new JCheckBox("ShowHorizontalLines");
		checkShowVerticalLines=new JCheckBox("ShowVerticalLines");
		textRowMargin=new JTextField();
		textRowMargin.setDocument(new IntegerDocument());
		textRowMargin.setBorder(createTitleBorder("RowMargin"));
		textColMargin=new JTextField();
		textColMargin.setDocument(new IntegerDocument());
		textColMargin.setBorder(createTitleBorder("ColMargin"));
		bHorizontalLinesColor=new JButton("horizontalLinesColor");
		bHorizontalLinesColor.setBorder(createTitleBorder("horizontalLinesColor"));
		bVerticalLinesColor=new JButton("verticalLinesColor");
		bVerticalLinesColor.setBorder(createTitleBorder("verticalLinesColor"));
		bBackground=new JButton("background");
		bForeground=new JButton("foreground");
		cFont=new ComboFontName();
		cFont.setBorder(createTitleBorder("Font"));
		cFontStyle=new ComboFontStyle();
		cFontStyle.setBorder(createTitleBorder("Style"));
		cFontSize=new ComboFontSize();
		cFontSize.setBorder(createTitleBorder("Size"));
		cTextPosition=new ComboHorizontalAlignment();
		cTextPosition.setBorder(createTitleBorder("Text Orientation"));
		textStyle=new JTextField() ;
		textStyle.setBorder(createTitleBorder("StyleFile"));
		//bStyle=new JButton("Style");
		bStyle=new JButton("...");
		bStyle.addActionListener(new ActionFileChooser(
				TableSetting.getFoderParent()));

		checkApplyStyles=new JCheckBox("ApplyStyles");
		bDefault =new JButton("Restore Defaults");
		bOK=new JButton("OK");
		bCancel=new JButton("Cancel");
	
		bHorizontalLinesColor.addActionListener(new ActionColorChooser());
		bVerticalLinesColor.addActionListener(new ActionColorChooser());
		bBackground.addActionListener(new ActionColorChooser());
		bForeground.addActionListener(new ActionColorChooser());
		checkApplyStyles.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent e) {
				textStyle.setEnabled(checkApplyStyles.isSelected());
				bStyle.setEnabled(checkApplyStyles.isSelected());
			}			
		});
		bDefault.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent e) {
				SettingPanel.this.activeSetting.loadDefault();
				tableSetting.setTableStyles(new TableStyles());
				tableSetting.setStylesFile(null);
				textStyle.setText("");
				checkApplyStyles.setSelected(true);
				confiSettingComponents();
			}
		});
		bOK.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent e) {
				accept=true;
				close();				
			}			
		});
		bCancel.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent e) {
				accept=false;
				close();				
			}			
		});
		
	}
	private Border createTitleBorder(String title){
		Border border=BorderFactory.createTitledBorder(title);		
		return border;
	}
	private void loadComponets() {
		loadComponets(tableSetting);
	}
	private void loadComponets(TableSetting tableSetting) {
		confiSettingComponents();
		if(tableSetting.getStylesFile()!=null){
			String sFile=Util.getRelativePath(
					tableSetting.getStylesFile(), TableSetting.getFoderParent());
			textStyle.setText(sFile);
		}
		checkApplyStyles.setSelected(tableSetting.isApplyStyles());
		textStyle.setEditable(false);
		textStyle.setEnabled(checkApplyStyles.isSelected());
		bStyle.setEnabled(checkApplyStyles.isSelected());
		
	}
	private void confiSettingComponents(){
		if(activeSetting!=null){
			checkShowHorizontalLines.setSelected(activeSetting.isShowHorizontalLines());
			checkShowVerticalLines.setSelected(activeSetting.isShowVerticalLines());
			textRowMargin.setText(Integer.toString(activeSetting.getRowMargin()));
			textColMargin.setText(Integer.toString(activeSetting.getColumnMargin()));
			bHorizontalLinesColor.setForeground(activeSetting.getHorizontalLinesColor());
			bVerticalLinesColor.setForeground(activeSetting.getVerticalLinesColor());
			bBackground.setBackground(activeSetting.getBackground());
			bForeground.setForeground(activeSetting.getForeground());
			
			Font font=activeSetting.getFont();
			if(font!=null){
				DefaultComboBoxModel model=(DefaultComboBoxModel) cFont.getModel();
				int size=model.getSize();
				for(int i=0; i<size; i++){
					Font f=(Font) model.getElementAt(i);
					if(font.getName().equals(f.getName())){
						cFont.setSelectedIndex(i);
						break;
					}
				}
				model=(DefaultComboBoxModel) cFontStyle.getModel();
				size=model.getSize();
				for(int i=0; i<size; i++){
					if(font.getStyle()==StyleSupport.value_style[i]){
						cFontStyle.setSelectedIndex(i);
						break;
					}
				}		
				model=(DefaultComboBoxModel) cFontSize.getModel();
				size=model.getSize();
				String sSize=Integer.toString(font.getSize());
				for(int i=0; i<size; i++){
					if(sSize.equals(StyleSupport.str_size[i])){
						cFontSize.setSelectedIndex(i);
						break;
					}
				}		
			}
			Integer htp=new Integer(activeSetting.getHorizontalTextAlignment());
			DefaultComboBoxModel model=(DefaultComboBoxModel) cTextPosition.getModel();
			int size=model.getSize();
			for(int i=0; i<size; i++){
				if(htp.equals(model.getElementAt(i))){
					cTextPosition.setSelectedIndex(i);
					break;
				}
			}					
		}
	}

	/**
	 * @return Returns the accept.
	 */
	public boolean isAccept() {
		return accept;
	}

	private void close(){
		JDialog fRoot=(JDialog)SwingUtilities
			.getRootPane(this).getParent();
		//fRoot.hide();
		fRoot.dispose();
	}
	private void loadActiveSetting() {
		activeSetting.setShowHorizontalLines(checkShowHorizontalLines.isSelected());
		activeSetting.setShowVerticalLines(checkShowVerticalLines.isSelected());
		int i=1;
		try {
			i=Integer.parseInt(textRowMargin.getText());
		} catch (NumberFormatException e) {
		}
		activeSetting.setRowMargin(i);
		try {
			i=Integer.parseInt(textColMargin.getText());
		} catch (NumberFormatException e) {
			i=1;
		}
		activeSetting.setColumnMargin(i);
		activeSetting.setHorizontalLinesColor(bHorizontalLinesColor.getForeground());
		activeSetting.setVerticalLinesColor(bVerticalLinesColor.getForeground());
		activeSetting.setBackground(bBackground.getBackground());
		activeSetting.setForeground(bForeground.getForeground());		
		Font font=(Font) cFont.getSelectedItem();
		int fontStyle=StyleSupport.value_style[cFontStyle.getSelectedIndex()];
		int fontSize=cFontSize.getSelectedIndex();
		try {
			fontSize=Integer.parseInt(StyleSupport.str_size[fontSize]);
		} catch (NumberFormatException e) {
			fontSize=10;
		}
		font=new Font(font.getName(),fontStyle,fontSize);
		activeSetting.setFont(font);
		activeSetting.setHorizontalTextAlignment(
				((Integer)cTextPosition.getSelectedItem()).intValue());
		
		if(tableSetting.getStylesFile()!=null){
			String sFile=Util.getRelativePath(
					tableSetting.getStylesFile(), TableSetting.getFoderParent());
			textStyle.setText(sFile);
		}
		tableSetting.setApplyStyles(checkApplyStyles.isSelected());
	}
	/**
	 * @return Returns the tableSetting.
	 */
	public TableSetting getTableSetting() {
		loadActiveSetting();
		return tableSetting;
	}
	
	private class ActionColorChooser extends AbstractAction{
		private static final long serialVersionUID = 1L;
		public void actionPerformed(ActionEvent e) {
			if(e.getSource() instanceof JButton){
				JButton b=(JButton) e.getSource();
				Color c;
				if(b==bBackground){
					c=b.getBackground();
				}else{
					c=b.getForeground();
				}
				c=JColorChooser.showDialog(b,b.getText(), c);
				if(c!=null){
					if(b==bBackground){
						b.setBackground(c);
					}else{
						b.setForeground(c);
					}
				}
			}			
		}
		
	}//End class ActionColorChooser
	private class ActionFileChooser extends AbstractAction{
		private static final long serialVersionUID = 1L;
		private File fParent;
		
		ActionFileChooser(File fParent) {
			super();
			if(fParent!=null){
				if(fParent.isDirectory()){
					this.fParent = fParent;
				}else{
					this.fParent=fParent.getParentFile();
				}
			}
		}

		public void actionPerformed(ActionEvent e) {
			if(e.getSource() instanceof JButton){
				JButton b=(JButton) e.getSource();
				JFileChooser fileChooser = new JFileChooser();
				fileChooser.setFileFilter(new StylesFilter());
				try{
					if (fParent!=null){
						fileChooser.setCurrentDirectory(fParent);
					}
				}catch(Exception ex){;}
				if(JFileChooser.APPROVE_OPTION ==fileChooser.showOpenDialog(b)){
					File f=fileChooser.getSelectedFile();
					if(tableSetting.setStylesFile(f)){
						String sFile=f.getAbsolutePath();
						if(fParent!=null){
							sFile=Util.getRelativePath(f, fParent);
						}
						textStyle.setText(sFile);
					}
				}
			}			
		}
		
	}// End class ActionFileChooser	
}// End class SettingPanel
