package misc.swing.table.settings;
import java.awt.Color;
import java.awt.Font;

import javax.swing.SwingConstants;
import javax.swing.UIManager;
import javax.swing.border.Border;
import misc.swing.border.BorderBuider;
import misc.swing.border.BordersInfo;

public class Style implements Cloneable{
	private String name;
	private Color background;
	private Color foreground;
	private Font font;
	private int horizontalAlignment;
	private int verticalAlignment;
	private boolean textAjustable;
	private int depth=0;
	private Border border;
	private BordersInfo bordersInfo;
	
	
	public Style(String name, boolean isHeader, boolean isRowHeader) {
		if(name!=null){
			this.name = name;
			loadDefault(isHeader, isRowHeader);
		}else{
			throw new IllegalArgumentException("name is null.");
		}
	}
	public Style(String name, boolean isHeader) {
		this(name, isHeader, false);
	}
	public Style(String name) {
		this(name,false, false);
	}
	private Style() {
	}
	public void loadDefault(boolean isHeader, boolean isRowHeader) {
		textAjustable=false;
		if(isHeader){
			background=UIManager.getColor("TableHeader.background");
			foreground=UIManager.getColor("TableHeader.foreground");
			font=UIManager.getFont("TableHeader.font");
			if( isRowHeader){
				horizontalAlignment=SwingConstants.LEADING;
			}else{
				horizontalAlignment=SwingConstants.CENTER;
				textAjustable=true;
			}
		}else{
			if(isRowHeader){
				background=UIManager.getColor("TableHeader.background");
				foreground=UIManager.getColor("TableHeader.foreground");
				font=UIManager.getFont("TableHeader.font");
				horizontalAlignment=SwingConstants.LEADING;
			}else{
				background=UIManager.getColor("Table.background");
				foreground=UIManager.getColor("Table.foreground");
				font=UIManager.getFont("Table.font");
				horizontalAlignment=SwingConstants.TRAILING;
			}
		}
		verticalAlignment=SwingConstants.CENTER;
	}
	/**
	 * @return Returns the background.
	 */
	public Color getBackground() {
		return background;
	}
	/**
	 * @param background The background to set.
	 */
	public void setBackground(Color background) {
		this.background = background;
	}
	/**
	 * @return Returns the font.
	 */
	public Font getFont() {
		return font;
	}
	/**
	 * @param font The font to set.
	 */
	public void setFont(Font font) {
		this.font = font;
	}
	/**
	 * @return Returns the foreground.
	 */
	public Color getForeground() {
		return foreground;
	}
	/**
	 * @param foreground The foreground to set.
	 */
	public void setForeground(Color foreground) {
		this.foreground = foreground;
	}
	/**
	 * @return Returns the horizontalAlignment.
	 */
	public int getHorizontalAlignment() {
		return horizontalAlignment;
	}
	/**
	 * @param horizontalAlignment The horizontalAlignment to set.
	 */
	public void setHorizontalAlignment(int horizontalAlignment) {
		this.horizontalAlignment = horizontalAlignment;
	}
	
	/**
	 * @return the verticalAlignment
	 */
	public int getVerticalAlignment() {
		return verticalAlignment;
	}
	/**
	 * @param verticalAlignment the verticalAlignment to set
	 */
	public void setVerticalAlignment(int verticalAlignment) {
		this.verticalAlignment = verticalAlignment;
	}
	/**
	 * @return Returns the name.
	 */
	public String getName() {
		return name;
	}
	/**
	 * @param name The name to set.
	 */
	public void setName(String name) {
		this.name = name;
	}
	/**
	 * @return Returns the depth.
	 */
	public int getDepth() {
		return depth;
	}
	/**
	 * @param depth The depth to set.
	 */
	public void setDepth(int depth) {
		this.depth = depth;
	}
	/**
	 * @return the border
	 */
	public Border getBorder() {
		if(border==null && bordersInfo!=null){
			border=BorderBuider.build(bordersInfo);
		}
		if(border!=null){
			return border;
		}
		return null;
	}
	/**
	 * @return the bordersInfo
	 */
	public BordersInfo getBordersInfo() {
		return bordersInfo;
	}
	/**
	 * @param borderInfo the bordersInfo to set
	 */
	public void setBordersInfo(BordersInfo bordersInfo) {
		this.bordersInfo = bordersInfo;
		if(bordersInfo!=null){
			border=BorderBuider.build(bordersInfo);
		}else{
			border=null;
		}
	}
	
	/**
	 * @return the textAjustable
	 */
	public boolean isTextAjustable() {
		return textAjustable;
	}
	/**
	 * @param textAjustable the textAjustable to set
	 */
	public void setTextAjustable(boolean textAjustable) {
		this.textAjustable = textAjustable;
	}
	/* (non-Javadoc)
	 * @see java.lang.Object#equals(java.lang.Object)
	 */
	public boolean equals(Object obj) {
		if(this==obj){
			return true;
		}
		if(obj instanceof Style){
			Style style=(Style)obj;
			if(style.name.equals(name) 
					&& style.background.equals(background)
					&& style.foreground.equals(foreground)
					&& style.horizontalAlignment==horizontalAlignment
					&& style.verticalAlignment==verticalAlignment
					&& style.depth==depth
					&& style.bordersInfo==bordersInfo
					&& style.textAjustable == textAjustable
					){
				return true;
			}
		}
		return false;
	}
	public Object clone(){
		Style style=new Style();
		style.name=name;
		style.background=background;
		style.foreground=foreground;
		style.font=font;
		style.horizontalAlignment=horizontalAlignment;
		style.verticalAlignment=verticalAlignment;
		style.depth=depth;
		style.bordersInfo=bordersInfo;
		style.textAjustable = textAjustable;
		return style;
	}
	
}
