package misc.swing.table.settings;



import java.awt.Color;
import java.awt.Font;

import misc.swing.border.BordersXMl;
import misc.xml.ColorXML;
import misc.xml.FontXML;
import org.jdom.Attribute;
import org.jdom.DataConversionException;
import org.jdom.Element;


public class StyleXML {
	public static final String KEY_STYLE="Style";
	private static final String KEY_NAME="Name";
	private static final String KEY_BACKGROUND="background";
	private static final String KEY_FOREGROUND="foreground";
	private static final String KEY_FONT="font";
	private static final String KEY_HORIZONTAL_TEXT_POSITION="horizontalTextAlignment";
	private static final String KEY_VERTICAL_TEXT_POSITION="verticalTextAlignment";
	private static final String KEY_TEXT_AJUSTABLE="textAjustable";
	private static final String KEY_DEPTH="depth";

	
	public static Style build(Element eStyle){
		if(eStyle!=null && KEY_STYLE.equals(eStyle.getName())){
			Style style=null;
			Attribute aName=eStyle.getAttribute(KEY_NAME);
			if(aName!=null){
				style=new Style(aName.getValue());
				Attribute aPosition=eStyle.getAttribute(KEY_HORIZONTAL_TEXT_POSITION);
				if(aPosition!=null){
					try {
						style.setHorizontalAlignment(aPosition.getIntValue());
					}catch (DataConversionException e) {
					}catch (RuntimeException e) {
					}
				}
				Attribute aVerPosition=eStyle.getAttribute(KEY_VERTICAL_TEXT_POSITION);
				if(aVerPosition!=null){
					try {
						style.setVerticalAlignment(aVerPosition.getIntValue());
					}catch (DataConversionException e) {
					}catch (RuntimeException e) {
					}
				}
				Attribute aAjustable=eStyle.getAttribute(KEY_TEXT_AJUSTABLE);
				if(aAjustable!=null){
					try {
						style.setTextAjustable(aAjustable.getBooleanValue());
					} catch (DataConversionException e) {
					}
				}
				Attribute aDepth=eStyle.getAttribute(KEY_DEPTH);
				if(aDepth!=null){
					try {
						style.setDepth(aDepth.getIntValue());
					}catch (DataConversionException e) {
					}catch (RuntimeException e) {
					}
				}
				Element eBackground=eStyle.getChild(KEY_BACKGROUND);
				Color c=null;
				if(eBackground!=null && eBackground.getChild(ColorXML.KEY_COLOR)!=null){
					c=ColorXML.build(eBackground.getChild(ColorXML.KEY_COLOR));
					if(c!=null){
						style.setBackground(c);
					}
				}
				Element eForeground=eStyle.getChild(KEY_FOREGROUND);
				c=null;
				if(eForeground!=null && eForeground.getChild(ColorXML.KEY_COLOR)!=null){
					c=ColorXML.build(eForeground.getChild(ColorXML.KEY_COLOR));
					if(c!=null){
						style.setForeground(c);
					}
				}				
				Element eFont=eStyle.getChild(KEY_FONT);
				Font f=null;
				if(eFont!=null && eFont.getChild(FontXML.KEY_FONT)!=null){
					f=FontXML.build(eFont.getChild(FontXML.KEY_FONT));
					if(f!=null){
						style.setFont(f);
					}
				}
				Element eBordesInfo=eStyle.getChild(BordersXMl.KEY_BORDERS_INFO);
				if(eBordesInfo!=null){
					style.setBordersInfo(BordersXMl.build(eBordesInfo));
				}
			}
			return style;
		}else{
			//throw new IllegalArgumentException("eStyle is null.");
			return null;
		}
	}
	public static Element getRootElement(Style style){
		if(style!=null){
			Element eStyle=new Element(KEY_STYLE);
			Attribute aName=new Attribute(KEY_NAME, style.getName());
			eStyle.setAttribute(aName);
			
			Attribute aPosition=new Attribute(KEY_HORIZONTAL_TEXT_POSITION
					, Integer.toString(style.getHorizontalAlignment()));
			eStyle.setAttribute(aPosition);

			Attribute aVerticalPosition=new Attribute(KEY_VERTICAL_TEXT_POSITION
					, Integer.toString(style.getVerticalAlignment()));
			eStyle.setAttribute(aVerticalPosition);
			
			Attribute aAjustable=new Attribute(KEY_TEXT_AJUSTABLE
					, Boolean.toString(style.isTextAjustable()));
			eStyle.setAttribute(aAjustable);

			Attribute aDepth=new Attribute(KEY_DEPTH
					, Integer.toString(style.getDepth()));
			eStyle.setAttribute(aDepth);
			
			Element eBackground= new Element(KEY_BACKGROUND);
			eBackground.addContent(ColorXML
					.getRootElement(style.getBackground()));
			eStyle.addContent(eBackground);
			
			Element eForeground= new Element(KEY_FOREGROUND);
			eForeground.addContent(ColorXML
					.getRootElement(style.getForeground()));
			eStyle.addContent(eForeground);
			
			Element eFont= new Element(KEY_FONT);
			eFont.addContent(FontXML
					.getRootElement(style.getFont()));
			eStyle.addContent(eFont);
			if(style.getBordersInfo()!=null){
				eStyle.addContent(BordersXMl.build(style.getBordersInfo()));
			}
			return eStyle;
		}else{
			//throw new IllegalArgumentException("style is null.");
			return null;
		}
	}
}
