package misc.swing.table.settings;



import java.io.File;
import misc.util.Util;


public class TableSetting {

	protected Setting dataSetting;
	protected Setting headerSetting;
	protected Setting rowHeaderSetting;
	private static File foderParent;
	protected File stylesFile;
	protected boolean applyStyles=true;
	
		
	/**
	 * 
	 * @param dataSetting
	 * @param headerSetting
	 * @param rowHeaderSetting
	 */
	public TableSetting(Setting dataSetting, Setting headerSetting, Setting rowHeaderSetting){
		if(dataSetting!=null){
			this.dataSetting = dataSetting;
		}else{
			this.dataSetting=new Setting(false);
		}
		if(headerSetting!=null){
			this.headerSetting = headerSetting;
		}else{
			this.headerSetting=new Setting(true);
		}
		if(rowHeaderSetting!=null){
			this.rowHeaderSetting = rowHeaderSetting;
		}else{
			this.rowHeaderSetting=new Setting(true,true);
		}
	}

	/**
	 * @param dataSetting
	 * @param headerSetting
	 */
	public TableSetting(Setting dataSetting, Setting headerSetting) {
		this(dataSetting, headerSetting, null);
	}
	public TableSetting() {
		this(null,null);
	}
	/**
	 * @return Returns the headerSetting.
	 */
	public Setting getHeaderSetting() {
		return headerSetting;
	}
	/**
	 * @param headerSetting The headerSetting to set.
	 */
	public void setHeaderSetting(Setting headerSetting) {
		this.headerSetting = headerSetting;
	}
	/**
	 * @return Returns the dataSetting.
	 */
	public Setting getDataSetting() {
		return dataSetting;
	}
	/**
	 * @param dataSetting The dataSetting to set.
	 */
	public void setDataSetting(Setting dataSetting) {
		this.dataSetting = dataSetting;
	}
	
	/**
	 * @return Returns the rowHeaderSetting.
	 */
	public Setting getRowHeaderSetting() {
		return rowHeaderSetting;
	}

	/**
	 * @param rowHeaderSetting The rowHeaderSetting to set.
	 */
	public void setRowHeaderSetting(Setting rowHeaderSetting) {
		this.rowHeaderSetting = rowHeaderSetting;
	}

	/**
	 * @return Returns the foderParent.
	 */
	public static File getFoderParent() {
		if(foderParent==null){
			foderParent=new File(".");
			foderParent=foderParent.getAbsoluteFile();
			if(foderParent.getParent()!=null){
				foderParent=foderParent.getParentFile();
			}
		}
		return foderParent;
	}
	/**
	 * @param foderParent The foderParent to set.
	 */
	public static void setFoderParent(File foderParent) {
		TableSetting.foderParent = foderParent;
	}
	/**
	 * @return Returns the stylesFile.
	 */
	public File getStylesFile() {
		return stylesFile;
	}


	/**
	 * @param stylesFile The stylesFile to set.
	 */
	public boolean setStylesFile(File stylesFile) {
		try {
			if(stylesFile!=null){
				TableStyles tableStyles=TableStylesXML.build(stylesFile);
				setTableStyles(tableStyles);
			}
			this.stylesFile = stylesFile;
			return true;
		} catch (RuntimeException e) {
		}
		return false;
	}
	public String getRelativePath(){
		if(stylesFile!=null){
			return Util.getRelativePath(stylesFile,getFoderParent());
		}
		return null;
	}
	/**
	 * Can return null
	 * @return Returns the tableStyles.
	 */
	public TableStyles getTableStyles() {
		TableStyles tableStyles=null;
		tableStyles=new TableStyles(headerSetting.getStyles()
				,dataSetting.getStyles()
				,rowHeaderSetting.getStyles());
		return tableStyles;
	}
	/**
	 * @param tableStyles The tableStyles to set.
	 */
	public void setTableStyles(TableStyles tableStyles) {
		if(tableStyles!=null){
			if(dataSetting!=null){
				dataSetting.setStyles(tableStyles.getDataStyles());
			}
			if(headerSetting!=null){
				headerSetting.setStyles(tableStyles.getHeaderStyles());
			}
			if(rowHeaderSetting!=null){
				rowHeaderSetting.setStyles(tableStyles.getRowHeaderStyles());
			}
		}
	}
//	public void clearStyles(){
//		if(dataSetting!=null){
//			dataSetting.setStyles(null);
//		}
//		if(headerSetting!=null){
//			headerSetting.setStyles(null);
//		}
//		if( rowHeaderSetting!=null){
//			rowHeaderSetting.setStyles(null);
//		}
//	}
	/**
	 * @return Returns the applyStyles.
	 */
	public boolean isApplyStyles() {
		return applyStyles;
	}
	/**
	 * @param applyStyles The applyStyles to set.
	 */
	public void setApplyStyles(boolean applyStyles) {
		this.applyStyles = applyStyles;
	}
	
	/* (non-Javadoc)
	 * @see java.lang.Object#equals(java.lang.Object)
	 */
	public boolean equals(Object obj) {
		if(obj==this){
			return true;
		}else if(obj instanceof TableSetting){
			TableSetting ts=(TableSetting)obj;
			if(ts.applyStyles!=applyStyles){
				return false;
			}
			if(!dataSetting.equals(ts.dataSetting)){
				return false;
			}
			if(!headerSetting.equals(ts.headerSetting)){
				return false;
			}
			if(!rowHeaderSetting.equals(ts.rowHeaderSetting)){
				return false;
			}
			if(ts.stylesFile!=null || stylesFile!=null){
				if(ts.stylesFile!=null && stylesFile!=null){
					if(!ts.stylesFile.equals(stylesFile)){
						return false;
					}
				}else{// uno es nulo
					return false;
				}
			}			
			return true;
		}
		return false;
	}

	public Object clone(){
		TableSetting tableSetting=
			new TableSetting((Setting)dataSetting.clone()
					, (Setting)headerSetting.clone()
					,(Setting)rowHeaderSetting.clone());
		tableSetting.stylesFile=stylesFile;
		tableSetting.applyStyles=applyStyles;
		tableSetting.setTableStyles((TableStyles) getTableStyles().clone());
		return tableSetting;
	}
}
