package misc.util;



import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.image.BufferedImage;
import java.awt.print.PageFormat;
import java.awt.print.Printable;
import javax.swing.JComponent;
import javax.swing.RepaintManager;

public class Printer {
	private JComponent component;

	
	private Printer(JComponent component) {
		super();
		this.component = component;
	}
	public static int printMonoPage(JComponent component, Graphics graphics, PageFormat pageFormat, int pageIndex) {
		return new Printer(component).printMonoPage(graphics, pageFormat, pageIndex);		
	}
	public static int printMultiPage(JComponent component, Graphics graphics, PageFormat pageFormat, int pageIndex) {
		return new Printer(component).printMultiPage(graphics, pageFormat, pageIndex);		
	}
	public static int printFullMonoPage(JComponent component, Graphics graphics, PageFormat pageFormat, int pageIndex) {
		return new Printer(component).printFullMonoPage(graphics, pageFormat, pageIndex);		
	}
	
	private BufferedImage m_bi = null;
	private int m_maxNumPage = 1;
	
	/**
	 * Implementacion de printable en una sola pagina (La imagen se trocea en varias pg);
	 * @param graphics
	 * @param pageFormat
	 * @param pageIndex
	 * @return
	 */
	private int printMultiPage(Graphics graphics, PageFormat pageFormat, int pageIndex) {
		Graphics2D g2d=(Graphics2D) graphics;
		if(m_bi==null){
			m_bi=createBufferedImage();
		}
		if (pageIndex < m_maxNumPage && m_bi != null){
			g2d.translate((int)pageFormat.getImageableX(), 
					(int)pageFormat.getImageableY());
			int wPage = (int)pageFormat.getImageableWidth();
			int hPage = (int)pageFormat.getImageableHeight();
			int w = m_bi.getWidth(component);
			int h = m_bi.getHeight(component);
			if (w > 0 && h > 0){
				int nCol = Math.max((int)Math.ceil((double)w/wPage), 1);
				int nRow = Math.max((int)Math.ceil((double)h/hPage), 1);
				m_maxNumPage = nCol*nRow;
				int iCol = pageIndex % nCol;
				int iRow = pageIndex / nCol;
				int x = iCol*wPage;
				int y = iRow*hPage;
				int wImage = Math.min(wPage, w-x);
				int hImage = Math.min(hPage, h-y);
				g2d.drawImage(m_bi, 0, 0, wImage, hImage, 
						x, y, x+wImage, y+hImage, component);
				//g2d.drawRect(0,0,wPage,hPage);
				System.gc();
				return Printable.PAGE_EXISTS;
			}
		}
		if(m_bi!=null){
			m_bi=null;
			System.gc();
		}
		return Printable.NO_SUCH_PAGE;
	}
	private  BufferedImage createBufferedImage(){
		BufferedImage buff=new BufferedImage(
				component.getWidth(), component.getHeight(),BufferedImage.TYPE_INT_RGB);
		Graphics g = buff.getGraphics();
		boolean initialDoubleBuffering=isDoubleBufferingEnabled();
		//Turn off double buffering
		if(initialDoubleBuffering){
			disableDoubleBuffering();
		}
		component.paint(g);
		//Turn on buffering
		if(initialDoubleBuffering){
			enableDoubleBuffering();
		}
		return buff;
	}
	/**
	 * Implementacion de printable en una sola pagina (La imagen no ocupa toda la pg)
	 * @param graphics
	 * @param pageFormat
	 * @param pageIndex
	 * @return
	 */
	private int printMonoPage(Graphics graphics, PageFormat pageFormat, int pageIndex) {
		if(pageIndex==0){
			boolean initialDoubleBuffering=isDoubleBufferingEnabled();
			Graphics2D g2d=(Graphics2D) graphics;
			g2d.translate(pageFormat.getImageableX(),pageFormat.getImageableY());
			//Turn off double buffering
			if(initialDoubleBuffering){
				disableDoubleBuffering();
			}
			int wPage = (int)pageFormat.getImageableWidth();
			int hPage = (int)pageFormat.getImageableHeight();
			double ex=1;
			double ey=1;
			if(wPage<component.getWidth()){
				ex=(double)wPage/component.getWidth();
			}
			if(hPage<component.getHeight()){
				ey=(double)hPage/component.getHeight();
			}
			g2d.scale(ex , ey);
			component.paint(g2d);
			if(initialDoubleBuffering){
				enableDoubleBuffering();
			}
			return Printable.PAGE_EXISTS;
		}
		return Printable.NO_SUCH_PAGE;
	}
	/**
	 * Implementacion de printable en una sola pagina (La imagen ocupa toda la pg)
	 * @param graphics
	 * @param pageFormat
	 * @param pageIndex
	 * @return
	 */
	private int printFullMonoPage(Graphics graphics, PageFormat pageFormat, int pageIndex) {
		if(pageIndex==0){
			boolean initialDoubleBuffering=isDoubleBufferingEnabled();
			Graphics2D g2d=(Graphics2D) graphics;
			g2d.translate(pageFormat.getImageableX(),pageFormat.getImageableY());
			g2d.scale(pageFormat.getImageableWidth()/component.getWidth()
					, pageFormat.getImageableHeight()/component.getHeight());
			//Turn off double buffering
			if(initialDoubleBuffering){
				disableDoubleBuffering();
			}
			component.paint(g2d);
			//Turn on buffering
			if(initialDoubleBuffering){
				enableDoubleBuffering();
			}
			return Printable.PAGE_EXISTS;
		}
		return Printable.NO_SUCH_PAGE;
	}
	
	private void disableDoubleBuffering(){
		RepaintManager currentManger=RepaintManager.currentManager(component);
		currentManger.setDoubleBufferingEnabled(false);
	}
	private void enableDoubleBuffering(){
		RepaintManager currentManger=RepaintManager.currentManager(component);
		currentManger.setDoubleBufferingEnabled(true);
	}
	private boolean isDoubleBufferingEnabled(){
		RepaintManager currentManger=RepaintManager.currentManager(component);
		return currentManger.isDoubleBufferingEnabled();
	}
	
}
