package misc.util;



import java.awt.Color;
import java.awt.Font;
import java.awt.FontMetrics;
import java.awt.Graphics;
import java.awt.print.PageFormat;
import java.awt.print.Printable;
import java.awt.print.PrinterException;

import javax.swing.JTable;
import javax.swing.UIManager;
import javax.swing.table.TableColumn;
import javax.swing.table.TableColumnModel;
import javax.swing.table.TableModel;

public class TablePrinter implements Printable{
	private Font tableFont =UIManager.getFont("Table.font");
	private Font headerFont =UIManager.getFont("TableHeader.font");//.deriveFont(Font.BOLD);
	private int m_maxNumPaginas = 1;
	private JTable table;
	
	public TablePrinter(JTable table) {
		super();
		this.table = table;
	}
	/**
	 * @see java.awt.print.Printable#print(java.awt.Graphics, java.awt.print.PageFormat, int)
	 */
	public int print(Graphics pg, PageFormat pageFormat, int indicePagina) throws PrinterException {
		try {
			if (indicePagina >= m_maxNumPaginas)
				return Printable.NO_SUCH_PAGE;
			pg.translate((int)pageFormat.getImageableX(),
					(int)pageFormat.getImageableY());
			int wPagina = 0;
			int hPagina = 0;
			wPagina = (int)pageFormat.getImageableWidth();
			hPagina = (int)pageFormat.getImageableHeight();
			TableModel tm=table.getModel();

			int y = 0;
			pg.setFont(headerFont);
			pg.setColor(Color.black);
			FontMetrics fm  = pg.getFontMetrics();

			int nColumns =tm.getColumnCount(); // total columnas
			if(nColumns==0)
				return Printable.NO_SUCH_PAGE;
			int x[] = new int[nColumns];
			int xLen[] = new int[nColumns];


			int h = fm.getAscent();

			// Paginas necesarias para pintar la fila entera
			int nPaginasPorFila=1;
			int nRow, nCol;
			TableColumnModel tcm=table.getColumnModel();
			TableColumn tc;
			for (nCol=0; nCol<nColumns; nCol++){
				tc=tcm.getColumn(nCol);

				//int width = tc.getWidth();
				int width=Math.max(fm.stringWidth(tc.getHeaderValue().toString()), tc.getWidth());
				//width +=2; //Margen
				width=Math.max(getMaxWidthCol(fm, nCol),width);
				if (width > wPagina) return Printable.NO_SUCH_PAGE;
				xLen[nCol]=width;
				if ((x[nCol] + width) > wPagina ){
					nPaginasPorFila++;
					x[nCol]=0;
				}
				if (nCol+1<nColumns) x[nCol+1] = x[nCol] + width;
			}
			//System.out.println("nPaginasPorFila = " + nPaginasPorFila);

			int colIniGrupo=0, colFinGrupo=0;
			int nGrupoActual=(indicePagina%nPaginasPorFila)+1;
			// Si para pintar una fila entera se necesitan varias hojas
			// =>estoy pintando, la hoja numero nGrupoActual.
			// Hay que cargar colIniGrupo y colFinGrupo
			for (int nGrupo=1; nGrupo <= nGrupoActual ; nGrupo++){
				for (nCol=colIniGrupo; nCol<nColumns; nCol++) {
					colFinGrupo= nCol;
					if (nCol+1<nColumns){
						if (x[nCol+1]==0){
							break;
						}
					}
				}
				if (nGrupo < nGrupoActual) colIniGrupo = colFinGrupo + 1 ;
			}
			// Pintado la cabecera de las table
			pg.setFont(headerFont);
			y += h;
			for (nCol=colIniGrupo; nCol<=colFinGrupo; nCol++) {
				tc=tcm.getColumn(nCol);
				//pg.drawString(tc.getHeaderValue().toString(), x[nCol]+(xLen[nCol]-fm.stringWidth(tc.getHeaderValue().toString()))/2, y);
				pg.drawString(tc.getHeaderValue().toString(),(x[nCol]+(xLen[nCol]-fm.stringWidth(tc.getHeaderValue().toString()))), y);
			}
			// Fin Pintado la cabecera de las table

			pg.setFont(tableFont);
			pg.setColor(Color.black);
			fm = pg.getFontMetrics();

			y+=5; //Espacio entre cabecera y Tabla
			int header = y;
			h = fm.getHeight();
			int rowH = Math.max(h, 10);
			int rowPorPagina = (hPagina-header)/rowH;
			m_maxNumPaginas = Math.max((int)Math.ceil(tm.getRowCount()/
					(double)rowPorPagina), 1) * nPaginasPorFila;

			int rowIniPagina = ((indicePagina-(nGrupoActual-1))/nPaginasPorFila)*rowPorPagina;
			int rowFinPagina = Math.min(tm.getRowCount(),rowIniPagina + rowPorPagina);
			//System.out.println("rowIniPagina= " + rowIniPagina+ " rowFinPagina= " + rowFinPagina));

			//Pintando las filas de la table
			for (nRow =rowIniPagina; nRow < rowFinPagina; nRow++) {
				y += h;
				for (nCol=colIniGrupo; nCol <= colFinGrupo ; nCol++) {
					Object o=tm.getValueAt(nRow, nCol);
					if(o!=null){
						String str=o.toString();
						pg.setFont(tableFont);
						pg.drawString(str,(x[nCol]+(xLen[nCol]-fm.stringWidth(str))), y);
					}
				}
			}


			//Se dibuja el margen solo para depurar.
			/* *
			pg.setFont(tableFont);
			pg.drawRect(0,0,wPagina,hPagina);
			* */

			System.gc();
			return Printable.PAGE_EXISTS;
		} catch (Exception e) {
			e.printStackTrace();
			return Printable.NO_SUCH_PAGE;
		}
	}
	private int getMaxWidthCol(FontMetrics fm, int indiceCol){
		int ancho=0;
		TableModel tm=table.getModel();
		for(int i=0; i<tm.getRowCount(); i++){
			Object o=tm.getValueAt(i, indiceCol);
			String str=(o!=null)? o.toString():"";
			ancho=Math.max(fm.stringWidth(str), ancho);
		}
		return ancho;
	}
	

}
